package net.rastro.superdroid;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.Base64Coder;
import com.badlogic.gdx.utils.Json;

public class SettingsManager {
	
	// the location of the profile data file
    private static final String SETTINGS_DATA_FILE = "data/settings.json";
	
	
	public static void load()
	{
		// create the handle for the profile data file
        FileHandle settingsDataFile = Gdx.files.local( SETTINGS_DATA_FILE );
        
     // create the JSON utility object
        Json json = new Json();
        if(settingsDataFile.exists())
        {
        	 // load the profile from the data file
            try {

                // read the file as text
                String settingsAsText = settingsDataFile.readString().trim();

                // decode the contents (if it's base64 encoded)
                if( settingsAsText.matches( "^[A-Za-z0-9/+=]+$" ) ) {
                    System.out.println("Settings are encoded, decoding as we speak" );
                    settingsAsText = Base64Coder.decodeString( settingsAsText );
                }

                // restore the state
                SuperDroid.settings = json.fromJson( Settings.class, settingsAsText );
                System.out.println("Settings Loaded");
                System.out.println(Settings.characterID);

            } catch( Exception e ) {

                System.out.println("Couldnt read the file, let's resave the defaults");
                save();
            }

        }
        else
        {
        	//if it doesnt exist, then save default values
        	save();
        }
        
        if(SuperDroid.DEVELOPER_MODE)
        	SuperDroid.settings.levelsFinished = 16;

	}
	
	public static void save()
	{
        // create the handle for the profile data file
        FileHandle settingsDataFile = Gdx.files.local( SETTINGS_DATA_FILE );
        
        if(settingsDataFile.exists())
        	System.out.println("Settings file found");

        // create the JSON utility object
        Json json = new Json();

        // convert the given profile to text
        String settingsAsText = json.toJson( SuperDroid.settings );
        
        System.out.println("Settings: " + settingsAsText);
        //String settingsAsText = "WTF";

        // encode the text
        if( ! SuperDroid.DEVELOPER_MODE ) {
        	settingsAsText = Base64Coder.encodeString( settingsAsText );
        }

        // write the profile data file
        settingsDataFile.writeString( settingsAsText, false );
        //json.writeValue( "currentLevelId", Settings.levelsFinished );        

    }

}
