package net.rastro.superdroid.controller;

import java.util.HashMap;
import java.util.Map;

import net.rastro.superdroid.Assets;
import net.rastro.superdroid.model.Apple;
import net.rastro.superdroid.model.Block;
import net.rastro.superdroid.model.Bob;
import net.rastro.superdroid.model.Coin;
import net.rastro.superdroid.model.Ladder;
import net.rastro.superdroid.model.Laser;
import net.rastro.superdroid.model.Lift;
import net.rastro.superdroid.model.MovingSpike;
import net.rastro.superdroid.model.World;
import net.rastro.superdroid.model.finishMark;
import net.rastro.superdroid.model.missile;

public class WorldController {

	enum Keys {
		LEFT, RIGHT, JUMP, FIRE ,UP, DOWN
	}

	private World 	world;
	private Bob 	bob;	
	private finishMark endSign;
	private float	jump;
	
	private float 	countDownDelta;
	
	private int surface = 0;
	private int surfacePlaying=-1;	
	
	public static boolean isSoundPlaying=false;
	
	boolean sidesLeft = false;
	boolean sidesRight = false;
	boolean ground = false;
	boolean head = false;
	
	boolean spikesHit = false;
	boolean spikesHitGlobal = false;
	boolean isFalling = false;

	static Map<Keys, Boolean> keys = new HashMap<WorldController.Keys, Boolean>();
	static {
		keys.put(Keys.LEFT, false);
		keys.put(Keys.RIGHT, false);
		keys.put(Keys.JUMP, false);
		keys.put(Keys.FIRE, false);
		keys.put(Keys.UP, false);
		keys.put(Keys.DOWN, false);
	};

	public WorldController(World world) {
		this.world = world;
		this.bob = world.getBob();
		this.endSign = world.getEnd();
	}

	// ** Key presses and touches **************** //

	public void upPressed() {
		keys.get(keys.put(Keys.UP, true));
	}
	
	public void downPressed() {
		keys.get(keys.put(Keys.DOWN, true));
	}
	
	public void leftPressed() {
		keys.get(keys.put(Keys.LEFT, true));
	}

	public void rightPressed() {
		keys.get(keys.put(Keys.RIGHT, true));
	}

	public void jumpPressed() {
		keys.get(keys.put(Keys.JUMP, true));
	}

	public void firePressed() {
		keys.get(keys.put(Keys.FIRE, true));
	}

	public void upReleased() {
		keys.get(keys.put(Keys.UP, false));
	}
	
	public void downReleased() {
		keys.get(keys.put(Keys.DOWN, false));
	}

	
	public void leftReleased() {
		keys.get(keys.put(Keys.LEFT, false));
	}

	public void rightReleased() {
		keys.get(keys.put(Keys.RIGHT, false));
	}

	public void jumpReleased() {
		keys.get(keys.put(Keys.JUMP, false));		
		if(bob.getState() == Bob.JUMPING)
		{
			bob.jump_speed=0;
		}
	}

	public void fireReleased() {
		keys.get(keys.put(Keys.FIRE, false));
	}

	/** The main update method **/
	public void update(float delta) {
		
		endSign.update(delta, bob.getBounds());
		//did bob fall out of the game bounadries?
		if(bob.getPosition().y < -5 && bob.getState() != Bob.DYING)
		{
			world.setGameState(World.GAME_LOST);
			bob.setState(Bob.DYING);
			Assets.playSound(Assets.deathSound);
			//stop the sounds
			Assets.dirtWalk.stop();
			Assets.stoneWalk.stop();
			Assets.stepIce.stop();
			
		}
			
		
		if(world.getGameState() == World.GAME_PAUSED)
		{
			blockCollision(delta);	
			//stop all sounds, I really need to make function for stopping all sound effect :D
			Assets.dirtWalk.stop();
			Assets.stoneWalk.stop();
			Assets.stepWood.stop();
			Assets.stepIce.stop();
			return;
		}
		
		if(world.getGameState() == World.GAME_READY)
		{
			updateCoins(delta);	
			blockCollision(delta);	
			//updateWalkingApples(delta);
			if(bob.jump_speed>0)
				bob.jump_speed=0;
			bob.getVelocity().x=0;			
			bob.update(delta);
			if(ground)
				bob.setState(Bob.IDLE);
			
			//stop all sounds, I really need to make function for stopping all sound effect :D
			Assets.dirtWalk.stop();
			Assets.stoneWalk.stop();
			Assets.stepWood.stop();
			Assets.stepIce.stop();
			return;
		}
		
		if(world.getGameState() == World.GAME_FINISH)
		{
			blockCollision(delta);	
			updateWalkingApples(delta);
			if(bob.jump_speed>0)
				bob.jump_speed=0;
			bob.getVelocity().x=0;			
			bob.update(delta);
			if(ground)
				bob.setState(Bob.IDLE);
			if(world.highScore == 0)
			{
				world.highScore = (world.countDownTimer+1)*world.coinsCollected;
			}
			
			//stop all sounds, I really need to make function for stopping all sound effect :D
			Assets.dirtWalk.stop();
			Assets.stoneWalk.stop();
			Assets.stepWood.stop();
			Assets.stepIce.stop();
			return;
		}
		
		if(bob.getState() != Bob.DYING)
		{				
			bob.dieTimeElapsed = 0;								
				processInput();				
			laserBeamCollision(delta);
			countDownDelta += delta;
			if(countDownDelta >= 1f)
			{
				world.countDownTimer -= 1;
				countDownDelta=0;
			}
			
		}		
		if(bob.dieTimeElapsed >=2.0)
		{			
			//System.out.println("Restart");
		}
		bob.dieTimeElapsed += delta;
		
		updateMissiles(delta);				
		blockCollision(delta);		
		updateLadders();
		updateMovingSpikes(delta);
		updateWalkingApples(delta);
		if(bob.getState() != Bob.DYING) //Crappy workaround
			updateLifts(delta);
		bob.update(delta);
		updateCoins(delta);		
	}
	
	public void updateWalkingApples(float delta)
	{
		boolean wasTheAppleDying = false;
		for(Apple apple : world.getApples())
		{			
			if(!apple.amIActive()) continue;
			
			wasTheAppleDying = apple.amIDying();
			apple.update(delta, bob.getBounds());
			if(bob.getState() == Bob.DYING) continue;
			if(apple.didIHitBob())
			{
				world.setGameState(World.GAME_LOST);
				bob.setState(Bob.DYING);	
				if(bob.jump_speed>0){								
					bob.jump_speed=0f;
					bob.getVelocity().y = 0;
				}
				Assets.playSound(Assets.deathSound);
				//stop the sounds
				Assets.dirtWalk.stop();
				Assets.stoneWalk.stop();
				Assets.stepIce.stop();
				break;
			}
			
			if(apple.bobOnHead() && !wasTheAppleDying && !apple.didIHitBob())
			{			
				if (keys.get(Keys.JUMP))									
					bob.jump_speed = 6.2f;
				else
					bob.jump_speed = 1.2f;
				Assets.playSound(Assets.appleSquash);
				bob.setState(Bob.JUMPING);				
				bob.getVelocity().y = Bob.jump_speed;
				world.countDownTimer += 10;
			}			
		}
	}
	
	public void updateMovingSpikes(float delta)
	{
		for(MovingSpike movingSpike : world.getMovingSpikes())
		{
			movingSpike.update(delta, bob.getBounds());
			
			if(movingSpike.didIHitBob() && bob.getState() != Bob.DYING)
			{
				world.setGameState(World.GAME_LOST);
				bob.setState(Bob.DYING);	
				if(bob.jump_speed>0){								
					bob.jump_speed=0f;
					bob.getVelocity().y = 0;
				}
				Assets.playSound(Assets.deathSound);
				//stop the sounds
				Assets.dirtWalk.stop();
				Assets.stoneWalk.stop();				
				break;
			}
		}
	}
	
	public void updateLifts(float delta)
	{
		bob.setRidingLift(false);
		for(Lift lift : world.getLifts())
		{			
			for(missile missile : world.getMissiles())
			{
				if(overlapTest.overlapRectangles(lift.getBounds(), missile.getBounds()) && missile.getState() == missile.STATE_FLYING)
				{
					missile.hitABlock();				
				}
			}
			lift.update(delta, bob.getBounds());
			if(lift.isBobRiding())
			{
				bob.getPosition().y = lift.getPosition().y + Lift.SIZE_H-0.1f;
				
				//System.out.println("STATE: " + bob.getState());
				if(bob.getState() == Bob.JUMPING || bob.getState() == bob.FALLING)
					bob.setState(Bob.IDLE);
				bob.getVelocity().y=0f;
				
				bob.setGround(true);
				isFalling = false;
				ground = true;
				surface = 0;
				bob.setRidingLift(true);
				
				if(keys.get(Keys.JUMP) && bob.getState() != Bob.DYING)
				{
					if(lift.isGoingUP())
						bob.jump_speed = Bob.JUMP_VELOCITY + Lift.SPEED-2f;
					else
						bob.jump_speed = Bob.JUMP_VELOCITY - Lift.SPEED+2f;					
					
					bob.getVelocity().y = Bob.jump_speed;
					bob.setState(Bob.JUMPING);
					bob.setRidingLift(false);
					bob.setGround(false);					
					ground = false;
				}
				
			}
		}
	}
	
	private void updateLadders()
	{
		bob.setLadder(false);
		for(Ladder ladder : world.getLadders())
		{
			ladder.update(bob.getBounds());
			if(ladder.isTouched() && bob.getState() != Bob.JUMPING)
			{
				bob.setLadder(true);
				//bob.setState(Bob.WALKING_LADDER);
				bob.setGround(true);
				surface = 3;
			}
		}
	}
	
	private void updateMissiles(float delta)
	{
		boolean amIDead = false;
		for(missile missile : world.getMissiles())
		{			
			missile.update(delta, bob.getBounds(), bob.getPosition());
			if(missile.getState() == missile.STATE_EXPLODING && missile.didIHitBob() == true) amIDead = true;
			if(amIDead && bob.getState() != Bob.DYING)
			{
				world.setGameState(World.GAME_LOST);
				bob.setState(Bob.DYING);	
				if(bob.jump_speed>0){								
					bob.jump_speed=0f;
					bob.getVelocity().y = 0;
				}
				Assets.playSound(Assets.deathSound);
				//stop the sounds
				Assets.dirtWalk.stop();
				Assets.stoneWalk.stop();				
				break;
			}
			
		}
	}
	
	private void updateCoins(float delta)
	{
		for(Coin coin : world.getCoins())
		{
			if(coin.isActive())
				coin.update(delta, bob.getBounds());
		}
	}
	
	private void laserBeamCollision(float delta){
		boolean amIDead = false;
		for(Laser laser : world.getLasers())
		{			
			laser.update(delta);
			//Let's check if bob has hit any lasers
			if(laser.isActive())
			{
				if(laser.getType() == Laser.GROUND)
				{
					if(bob.getPosition().x <= laser.getPosition().x+Laser.SIZE/2 && 
							bob.getPosition().x+bob.getBounds().width >= laser.getPosition().x+Laser.SIZE/2 &&
							bob.getPosition().y >= laser.getPosition().y && bob.getPosition().y <= laser.getPosition().y+laser.getLength())
					{
						//System.out.println("Laser HIT");
						amIDead=true;						
					}
				}
				
				if(laser.getType() == Laser.CEILING)
				{
					if(bob.getPosition().x <= laser.getPosition().x+Laser.SIZE/2 && 
							bob.getPosition().x+bob.getBounds().width >= laser.getPosition().x+Laser.SIZE/2 &&
							bob.getPosition().y <= laser.getPosition().y && bob.getPosition().y >= laser.getPosition().y-laser.getLength())
					{
						amIDead=true;
					}
				}
				
				if(laser.getType() == Laser.LEFT)
				{
					if(bob.getPosition().y <= laser.getPosition().y+Laser.SIZE/2 && 
							bob.getPosition().y+bob.getBounds().height >= laser.getPosition().y+Laser.SIZE/2 &&
							bob.getPosition().x >= laser.getPosition().x && bob.getPosition().x <= laser.getPosition().x+laser.getLength())
					{
						amIDead=true;
					}
				}
				
				if(laser.getType() == Laser.RIGHT)
				{
					if(bob.getPosition().y <= laser.getPosition().y+Laser.SIZE/2 && 
							bob.getPosition().y+bob.getBounds().height >= laser.getPosition().y+Laser.SIZE/2 &&
							bob.getPosition().x <= laser.getPosition().x && bob.getPosition().x >= laser.getPosition().x-laser.getLength())
					{
						amIDead=true;
					}
				}
			}
			if(amIDead)
			{
				world.setGameState(World.GAME_LOST);
				bob.setState(Bob.DYING);	
				if(bob.jump_speed>0){								
					bob.jump_speed=0f;
					bob.getVelocity().y = 0;
				}
				Assets.playSound(Assets.deathSound);
				//stop the sounds
				Assets.dirtWalk.stop();
				Assets.stoneWalk.stop();				
				break;
			}
		}
	}

	private void blockCollision(float delta) {
		sidesLeft = false;
		sidesRight =false;
		ground = false;
		spikesHitGlobal = false;
		bob.setGround(false);
		bob.iceIceBaby(false);
		if(bob.isBobRidingLift())
		{
			bob.setGround(true);
			ground = true;
		}
		for (Block block : world.getcollidableBlocks()) {
			block.updateBlock(delta);
			if(!block.amIActive()) continue;
			//Check for missiles
			for(missile missile : world.getMissiles())
			{
				if(overlapTest.overlapRectangles(block.getBounds(), missile.getBounds()) && missile.getState() == missile.STATE_FLYING)
	    		{
					missile.hitABlock();
//					System.out.println(block.getPosition() + " " + missile.getPosition() + " " + missile.getBounds());
	    		}
			}
			//Check for Bob
			spikesHit = false;
			
			/*if(overlapTest.overlapRectangles(bob.getBounds(), block.getBounds()))
				System.out.println("Hello, Earthling");*/
			if(overlapTest.groundCheck(bob.getBounds(), block.getBounds()))
			{
				//System.out.println("Ground");
				bob.getPosition().y = block.getPosition().y+block.getSizeY();
				ground = true;
				spikesHit = true;
				bob.setGround(true);	
				surface = block.getType();
				if(block.getType() == Block.GRASS_DISOLVE)
				{
					surface = Block.GRASS;
					block.dissolve();
				}
				if(block.getType() == Block.DIRT_DISSOLVE)
				{
					surface = Block.DIRT;
					block.dissolve();
				}
				if(block.getType() == Block.STONE_DISSOLVE)
				{
					surface = Block.STONE;
					block.dissolve();
				}
				if(block.getType() == Block.ICE)
				{
					bob.iceIceBaby(true);					
				}
				if(block.getType() == Block.ICE_DISSOLVE)
				{
					bob.iceIceBaby(true);				
					block.dissolve();
					surface = Block.ICE;
				}
			}				
			
			
			if(overlapTest.leftCheck(bob.getBounds(), block.getBounds()))
			{
				//System.out.println("Left");
				sidesLeft = true;		
				spikesHit = true;
				if(keys.get(Keys.LEFT) || bob.getVelocity().x < 0f)
				{
					bob.getVelocity().x = 0;
					//bob.getPosition().x = block.getPosition().x+block.SIZE;
				}				
			}
			if(overlapTest.rightCheck(bob.getBounds(), block.getBounds()))
			{
				//System.out.println("Right");
				sidesRight = true;
				spikesHit = true;
				spikesHitGlobal = true;
				if(keys.get(Keys.RIGHT) || bob.getVelocity().x > 0f)	
				{
					bob.getVelocity().x = 0;
					//bob.getPosition().x = block.getPosition().x-bob.SIZE;
				}
				
			}
			
			
			if(overlapTest.headCheck(bob.getBounds(), block.getBounds()))
			{
				//System.out.println("Head");
				//bob.getPosition().y = block.getPosition().y+bob.SIZE;
				bob.getPosition().y = block.getPosition().y-bob.BOB_HEIGHT-0.001f;
				head = true;
				spikesHit = true;
				bob.getVelocity().y = 0;
				bob.jump_speed = 0;
				
				if(block.getType() == Block.GRASS_DISOLVE || block.getType() == Block.DIRT_DISSOLVE || 
						block.getType() == Block.STONE_DISSOLVE || block.getType() == Block.ICE_DISSOLVE)
				{
					block.dissolve();
				}
	
			}	
			if(block.getType() == Block.SPIKES && spikesHit && bob.getState() != Bob.DYING)
			{
					//bob.jump_speed = 2f;
					//bob.getVelocity().y = bob.jump_speed;
					bob.setState(Bob.DYING);				
					world.setGameState(World.GAME_LOST);
						bob.getVelocity().x = 0;
						bob.getVelocity().y = 0;
						spikesHitGlobal= true;
												
						Assets.playSound(Assets.deathSound);
						
						//stop the sounds
						Assets.dirtWalk.stop();
						Assets.stoneWalk.stop();
						break;
						
			}
			//System.out.println(block.getBounds() + " " + bob.getBounds());
		}		
		
		if(keys.get(Keys.LEFT) && sidesLeft && sidesRight && bob.esSlidejuPaSlidotavu())
			bob.getVelocity().x = -bob.SPEED;
		if(keys.get(Keys.RIGHT) && sidesRight && sidesLeft && bob.esSlidejuPaSlidotavu())
			bob.getVelocity().x = bob.SPEED;
		
		if(!ground && !bob.getLadder() && !bob.isBobRidingLift())
		{
			if(bob.getState() != Bob.JUMPING && bob.getState() != Bob.FALLING &&  bob.getState() != Bob.DYING)
			{
				bob.jump_speed = 0f;
				bob.setState(Bob.JUMPING);				
				bob.getVelocity().y = Bob.jump_speed;
			}
		}
	}

	/** Change Bob's state and parameters based on input controls **/
	private void processInput() {
		
		if(keys.get(Keys.UP) && bob.getLadder()  && !keys.get(Keys.DOWN))
		{
			bob.getVelocity().y = Bob.SPEED;
			bob.setState(Bob.WALKING_LADDER);
			isFalling = false;
			ground = true;
			bob.setGround(true);
			bob.setState(Bob.WALKING_LADDER);
		}
		if(!keys.get(Keys.UP) && bob.getLadder() && !keys.get(Keys.DOWN))
		{
			bob.getVelocity().y = 0;
			bob.setState(Bob.WALKING_LADDER);
			isFalling = false;
			ground = true;
			bob.setGround(true);
			bob.setState(Bob.WALKING_LADDER);
		}
		
		if(keys.get(Keys.DOWN) && bob.getLadder() && !ground && !keys.get(Keys.UP))
		{
			bob.getVelocity().y = -Bob.SPEED;
			bob.setState(Bob.WALKING_LADDER);
			isFalling = false;
			ground = true;
			bob.setGround(true);
			bob.setState(Bob.WALKING_LADDER);
		}
		if(!keys.get(Keys.DOWN) && bob.getLadder() && !keys.get(Keys.UP))
		{
			bob.getVelocity().y = 0;
			bob.setState(Bob.WALKING_LADDER);
			isFalling = false;
			ground = true;
			bob.setGround(true);
			bob.setState(Bob.WALKING_LADDER);
		}
		
		if(bob.getPosition().x <=0 && keys.get(Keys.LEFT))
		{			
			bob.setFacingLeft(true);
			sidesLeft = true;
			bob.getVelocity().x = 0;
		}
		if (keys.get(Keys.LEFT)  && !sidesLeft) {
			// left is pressed
			bob.setFacingLeft(true);
			if(bob.getState()!=Bob.JUMPING && bob.getState()!=Bob.FALLING)
				bob.setState(Bob.WALKING);			
			if(!bob.esSlidejuPaSlidotavu())			
				bob.getVelocity().x = -Bob.SPEED;
			else if(bob.getVelocity().x > -Bob.SPEED-1f)
				bob.getVelocity().x -= 0.06f;
		}
		if (keys.get(Keys.RIGHT) && !sidesRight) {
			// left is pressed
			bob.setFacingLeft(false);
			if(bob.getState()!=Bob.JUMPING && bob.getState()!=Bob.FALLING)
				bob.setState(Bob.WALKING);
			if(!bob.esSlidejuPaSlidotavu())				
				bob.getVelocity().x = Bob.SPEED;
			else if(bob.getVelocity().x < Bob.SPEED+1f)
				bob.getVelocity().x += 0.06f;
		}
		if (keys.get(Keys.JUMP) && !isFalling) {
			// jump is pressed	
			if(bob.getState() != Bob.JUMPING && bob.getState() != Bob.FALLING)
			{
				bob.jump_speed = 6.2f;
				Assets.playSound(Assets.jumpSound);

				bob.setState(Bob.JUMPING);				
				bob.getVelocity().y = Bob.jump_speed;
			}
			
		}
		
		
		if (keys.get(Keys.FIRE)) {
			// jump is pressed			
				//jump = bob.getPosition().y;
				bob.getVelocity().y = -Bob.JUMP_VELOCITY;			
		}
		// need to check if both or none direction are pressed, then Bob is idle
		if((keys.get(Keys.LEFT) && keys.get(Keys.RIGHT)) || (!keys.get(Keys.LEFT) && !keys.get(Keys.RIGHT))){
			if(bob.getState()!=Bob.JUMPING && bob.getState()!=Bob.FALLING)
				if((keys.get(Keys.UP) && keys.get(Keys.DOWN)) || (!keys.get(Keys.UP) && !keys.get(Keys.DOWN)))
				bob.setState(Bob.IDLE);
			// acceleration is 0 on the x	
			if(!bob.esSlidejuPaSlidotavu())
			{
				bob.getAcceleration().x = 0;
				//horizontal speed is 0
				bob.getVelocity().x = 0;
			}
			//System.out.println("fuckyou");
		}
				
		if(bob.getState() == Bob.WALKING || bob.getState() == Bob.WALKING_LADDER)
			bob.setBobWalking(true); 
		else
			bob.setBobWalking(false);
		
		
		//System.out.println("surface: " + surface);
		
		if((!bob.isBobWalking()  && isSoundPlaying) || surfacePlaying != surface )
		{
			if(surfacePlaying == 0)
				Assets.stoneWalk.stop();
			if(surfacePlaying == 1 || surfacePlaying == 2)
				Assets.dirtWalk.stop();
			if(surfacePlaying == 3)
				Assets.stepWood.stop();
			if(surfacePlaying == Block.ICE)
				Assets.stepIce.stop();
			isSoundPlaying = false;			
		}
				
		
		if((bob.isBobWalking() && !isSoundPlaying) || surfacePlaying != surface){	
			if(surface==0)
			{
				surfacePlaying = surface;
				Assets.loopSound(Assets.stoneWalk);
			}
			if(surface==1 || surface==2)
			{
				surfacePlaying = surface;
				Assets.loopSound(Assets.dirtWalk);
			}
			if(surface == 3)
			{
				surfacePlaying = surface;
				Assets.loopSound(Assets.stepWood);
			}
			if(surface == Block.ICE)
			{
				surfacePlaying = surface;
				Assets.loopSound(Assets.stepIce);
			}
			isSoundPlaying = true;
		}
				
		
		//if(!keys.get(Keys.JUMP) && !keys.get(Keys.FIRE)) bob.getVelocity().y = 0;			
	}
}