package net.rastro.superdroid.model;

import net.rastro.superdroid.controller.overlapTest;

import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;

public class Apple {

	public static final float SIZE = 1f;
	public static final float SPEED = 1.5f;
	public static final float DIE_TIME = 0.6f;
	private float stateTime = 0;

	Vector2 	position = new Vector2();
	Vector2		startPos = new Vector2();
	Vector2		endPos = new Vector2();
	Vector2 	velocity = new Vector2();
	Rectangle 	bounds = new Rectangle();	
	boolean 	hitBob=false;
	boolean		isActive=true;
	boolean		walkingLeft=true;
	boolean		amIDying = false;
	boolean		bobIsOnMyHead = false;
	
	public Apple(Vector2 startPos, Vector2 endPos) {
		this.position.set(startPos);
		this.startPos.set(startPos);
		this.endPos.set(endPos);
		this.velocity.set(-this.SPEED, 0f);
		this.bounds = new Rectangle(startPos.x, startPos.y, SIZE, SIZE);		
	}
	
	public void update(float delta, Rectangle bobBounds)
	{
		if(!isActive) return;
		
		stateTime+=delta;		
		
		if(amIDying)
		{
			if(stateTime >= this.DIE_TIME) isActive = false; 
			return;
		}
		
		if(this.position.x >= this.startPos.x && walkingLeft)
		{
			walkingLeft = false;
			this.velocity.x=-this.SPEED;			
		}
		if(this.position.x <= this.endPos.x && !walkingLeft)
		{
			walkingLeft = true;
			this.velocity.x=this.SPEED;		
		}
		
		
		position.add(velocity.tmp().mul(delta));
		this.bounds = new Rectangle(position.x , position.y, SIZE, SIZE);		
		//System.out.println("Apple position: " + this.position + " " + endPos + " " + startPos + " " + bobBounds.y);
				
		Rectangle tempRect = new Rectangle(position.x+0.3f, position.y+0.7f, SIZE-0.6f, SIZE-0.7f);
		if(overlapTest.overlapRectangles(bobBounds, tempRect ))
		{
			//System.out.println("Apple top touched");
			amIDying = true;
			stateTime = 0f;
			bobIsOnMyHead = true;
		}
		
		tempRect = new Rectangle(position.x+0.3f, position.y, SIZE-0.6f, SIZE-0.5f);
		if(overlapTest.overlapRectangles(bobBounds, tempRect ))
		{
			//System.out.println("Apple killed BOB");
			amIDying = false;
			bobIsOnMyHead = false;
			hitBob=true;
		}
		
	}
	
	
	public Vector2 getPosition() {
		return position;
	}

	public Rectangle getBounds() {
		return bounds;
	}	
	
	public float getTime()
	{
		return stateTime;
	}
	
	public void resetPosition()
	{
		this.position.set(startPos);
		this.hitBob = false;		
		this.isActive=true;
		this.walkingLeft=true;
		this.amIDying = false;
		this.bobIsOnMyHead = false;
	}
	public boolean didIHitBob()
	{
		return hitBob;
	}
	
	public boolean amIDying()
	{
		return amIDying;
	}
	
	public boolean amIActive()
	{
		return isActive;
	}
	
	public boolean bobOnHead()
	{
		return bobIsOnMyHead;
	}
	
	public boolean isWalkingLeft()
	{
		return walkingLeft;
	}
}