package net.rastro.superdroid.model;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Array;
import net.rastro.superdroid.controller.overlapTest;

public class World {
	
	static int EMPTY = 0;
    static int STONE_TILE = 0x404040;
    static int GRASS_TILE = 0x00ff00;
    static int DIRT_TILE = 0x4f2500;
    static int SIGN_POLE = 0xffffff;
    static int START = 0xff0000;
    static int END = 0xff00ff;
    static int SPIKES_TILE = 0xffff00;
    static int LASER = 0x3600ff;
    static int LASER_PULSE = 0x00ccff;
    static int COIN = 0xff7200;
    static int ROCKET_LAUNCHER = 0xfeafff;
    static int LADDERS = 0xfff775;
    static int LIFT = 0xcab897;
    static int MOVING_SPIKE = 0x737300;
    static int GRASS_DISSOLVING = 0x51f851;
    static int DIRT_DISSOLVING = 0xb05200;
    static int STONE_DISSOLVING = 0x8b7868;
    static int ICE_TILE = 0x82a5ff;
    static int ICE_DISSOLVING = 0xcfddff;
    static int WALKING_APPLE = 0xff0090;       
    static int SIGN_LEFT = 0xb400d5;
    static int SIGN_UP = 0x40004c;
    static int SIGN_DOWN = 0x7f0000;
    
    public static final int GAME_LOADING = 0;
    public static final int GAME_RUNNING = 1;
    public static final int GAME_FINISH = 2;
    public static final int GAME_LOST = 3;
    public static final int GAME_READY = 4;
    public static final int GAME_PAUSED = 5;
    
    
    public static int coinsCollected=0;    
    public static int countDownTimer = 500;
    public static int highScore = 0;
    public static int coinCount = 0;
    public static int starEstimatedPoints = 0;



	/** The blocks making up the world **/
	Array<Block> blocks = new Array<Block>();
	Array<Block> collidableBlocks = new Array<Block>();
	Array<Block> backgroundBlocks = new Array<Block>();
	Array<gosign> gosignA = new Array<gosign>();
	Array<Coin> coins = new Array<Coin>();
	Array<Laser> lasers = new Array<Laser>();
	Array<rockets> rockets = new Array<rockets>();
	Array<missile> missiles = new Array<missile>();
	Array<Ladder> ladders = new Array<Ladder>();
	Array<Lift> lifts = new Array<Lift>();
	Array<MovingSpike> movingSpikes = new Array<MovingSpike>();
	Array<Apple> apples = new Array<Apple>();
	
	int[][] tilesXY;	
	
	static int game_state;
	/** Our player controlled hero **/
	Bob bob;
	finishMark endSign;
	boolean amIAlone;
	
	public static float WORLD_WIDTH = 150;
	public static float WORLD_HEIGHT = 35;
	// Getters -----------
	public Array<Block> getBackgroundBlocks() {
		return backgroundBlocks;
	}
	public Array<Block> getBlocks() {
		return blocks;
	}
	public Array<Apple> getApples()
	{
		return apples;
	}
	public Array<MovingSpike> getMovingSpikes() {
		return movingSpikes;
	}
	public Array<Block> getcollidableBlocks() {
		return collidableBlocks;
	}
	public Array<gosign> getGoSign() {
		return gosignA;
	}
	public Array<Lift> getLifts() {
		return lifts;
	}
	public Array<rockets> getRockets() {
		return rockets;
	}
	public Array<Coin> getCoins() {
		return coins;
	}
	public Array<Ladder> getLadders() {
		return ladders;
	}
	public Array<missile> getMissiles() {
		return missiles;
	}
	
	public Array<Laser> getLasers() {
		return lasers;
	}
	public Bob getBob() {
		return bob;
	}
	public finishMark getEnd(){
		return endSign;
	}
	// --------------------

	int currentLevel;
	
	public World(int levelId) {
		//createDemoWorld();
		highScore = 0;
		coinCount = 0;
		countDownTimer = 500;
		coinsCollected=0;
		currentLevel = levelId;
		readWorldFromfile();
	}
	
	private void readWorldFromfile()
	{		

		Pixmap pixmap = new Pixmap(Gdx.files.internal("levels/level"+currentLevel+".png"));
		tilesXY = new int[pixmap.getWidth()][pixmap.getHeight()];

		//Generate map blocks
		for (int y = 0; y < 35; y++) {
            for (int x = 0; x < 150; x++) {                         
                    int pix = (pixmap.getPixel(x, y) >>> 8) & 0xffffff;
                    Gdx.app.log("Map", x + ", " + y + ", " + Integer.toHexString(pix));
                    if (match(pix, START)) {                    	
                    	bob = new Bob(new Vector2(x, pixmap.getHeight() - 1 - y));
                    	//bob.spawnPosition.set(x, pixmap.getHeight() - y);
                    	bob.spawnX = x;
                    	bob.spawnY = pixmap.getHeight() - 1 - y + 0.01f;
                		bob.setFacingLeft(false);                		
                		//System.out.println("ASSHOLE ASSHOLE ASSHOLE");
                    } else if (match(pix, STONE_TILE)) {                    	
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 1;                    	
                        blocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.STONE));
                    } else if (match(pix, GRASS_TILE)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 1;
                    	blocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.GRASS));
                    } else if (match(pix, DIRT_TILE)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 1;
                    	blocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.DIRT));
                    } else if (match(pix, SPIKES_TILE)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 2;
                    	blocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.SPIKES));
                    } else if (match(pix, GRASS_DISSOLVING)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 2;
                    	blocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.GRASS_DISOLVE));
                    } else if (match(pix, DIRT_DISSOLVING)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 2;
                    	blocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.DIRT_DISSOLVE));
                    } else if (match(pix, STONE_DISSOLVING)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 2;
                    	blocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.STONE_DISSOLVE));
                    } else if (match(pix, ICE_TILE)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 1;
                    	blocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.ICE));
                    } else if (match(pix, ICE_DISSOLVING)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 2;
                    	blocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.ICE_DISSOLVE));
                    } else if(match(pix, SIGN_POLE)) {
                    	gosignA.add(new gosign(new Vector2(x, pixmap.getHeight() - 1 - y), gosign.Types.RIGHT));
                    } else if(match(pix, SIGN_LEFT)) {
                    	gosignA.add(new gosign(new Vector2(x, pixmap.getHeight() - 1 - y), gosign.Types.LEFT));
                    } else if(match(pix, SIGN_UP)) {
                    	gosignA.add(new gosign(new Vector2(x, pixmap.getHeight() - 1 - y), gosign.Types.UP));
                    } else if(match(pix, SIGN_DOWN)) {
                    	gosignA.add(new gosign(new Vector2(x, pixmap.getHeight() - 1 - y), gosign.Types.DOWN));
                    } else if(match(pix, LASER)) {
                    	lasers.add(new Laser(new Vector2(x, pixmap.getHeight() - 1 - y), true));
                    } else if(match(pix, LASER_PULSE)) {
                    	lasers.add(new Laser(new Vector2(x, pixmap.getHeight() - 1 - y), false));
                    } else if(match(pix, COIN)) {
                    	this.coinCount++;
                    	coins.add(new Coin(new Vector2(x, pixmap.getHeight() - 1 - y)));
                    } else if(match(pix, END)) {
                    	endSign = new finishMark(new Vector2(x, pixmap.getHeight() - 1 - y));
                    } else if(match(pix, ROCKET_LAUNCHER)) {
                    	rockets.add(new rockets(new Vector2(x, pixmap.getHeight() - 1 - y)));
                    	missiles.add(new missile(new Vector2(x+0.5f, pixmap.getHeight() - 1 - y-0.5f), new Vector2(x, pixmap.getHeight() - 1 - y)));
                    } else if(match(pix, LADDERS)) {
                    	ladders.add(new Ladder(new Vector2(x, pixmap.getHeight() - 1 - y)));                    	
                    } else if(match(pix, LIFT)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 15;                	
                    } else if(match(pix, MOVING_SPIKE)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 20;              	
                    } else if(match(pix, WALKING_APPLE)) {
                    	tilesXY[x][pixmap.getHeight() - 1 - y] = 25;                    	
                    }
                    

            }
		}
		
		Pixmap backgroundPixmap = new Pixmap(Gdx.files.internal("levels/level"+currentLevel+"b.png"));		

		//Generate map blocks
		for (int y = 0; y < 35; y++) {
            for (int x = 0; x < 150; x++) {                         
                    int pix = (backgroundPixmap.getPixel(x, y) >>> 8) & 0xffffff;
                    Gdx.app.log("Background Map", x + ", " + y + ", " + Integer.toHexString(pix));
                    if (match(pix, STONE_TILE)) {                    	                    	                    
                    	backgroundBlocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.STONE));                    
                    } else if (match(pix, DIRT_TILE)) {                    	
                    	backgroundBlocks.add(new Block(new Vector2(x, pixmap.getHeight() - 1 - y), Block.DIRT));
                    }                                       
            }
		}
		
		//Determine lifts and their cooridnates
		boolean foundStartPos, foundSecond;
		Vector2 startPos = new Vector2();
		Vector2 endPos = new Vector2();
		for(int x = 0; x<WORLD_WIDTH; x++)
		{
			foundStartPos = false;
			foundSecond = false;
			
			for(int y = 0; y<WORLD_HEIGHT; y++)
			{
				if(tilesXY[x][y] == 15 ) //20 is just a random number chosen by me to represent LIFT tile
				{					
					if(foundStartPos == false)
					{
						startPos.set(x,y);
						foundStartPos = true;						
					}
					else
					{
						endPos.set(x,y);
						foundSecond = true;						
					}					
				}
			}
			
			if(foundSecond)
			{
				lifts.add(new Lift(startPos, endPos));     
				System.out.println("Lift found: " + startPos + " " + endPos);
			}
		}
		
		//Calculate moving spikes
		for(int x = 0; x<WORLD_WIDTH; x++)
		{
			
			foundStartPos = false;			
			
			for(int y = (int) WORLD_HEIGHT-1; y>=0; y--)
			{
				if(tilesXY[x][y] == 20)
				{
					foundStartPos = true;	
					startPos.set(x,y);
				}
				if((foundStartPos) && (tilesXY[x][y] == 1 || tilesXY[x][y] == 2))
				{
					endPos.set(x,y+1);
					foundStartPos = false;			
					movingSpikes.add(new MovingSpike(startPos, endPos));					
				}
			}
		}
		
		//Calculate walking apple
				for(int y = (int) WORLD_HEIGHT-1; y>=0; y--)
				{
					
					foundStartPos = false;			
					
					for(int x = (int) WORLD_WIDTH-1; x>=0; x--)
					{						
						if((foundStartPos) && (tilesXY[x][y] == 1 || tilesXY[x][y] == 2 || tilesXY[x][y-1] != 1))
						{
							endPos.set(x+1,y);
							foundStartPos = false;			
							apples.add(new Apple(startPos, endPos));					
						}
						if(tilesXY[x][y] == 25)
						{
							foundStartPos = true;	
							startPos.set(x,y);
						}
					}
				}
		
		//Determine which blocks are collidable		
		for(Block block : blocks)
		{
			amIAlone = false;
			int x = (int) block.getPosition().x;
			int y = (int) block.getPosition().y;
			if(x-1 > 0)
				if(tilesXY[x-1][y] != 1) amIAlone = true;
			if(x+1 < WORLD_WIDTH)
				if(tilesXY[x+1][y] != 1) amIAlone = true;
			if(y-1>0)				
				if(tilesXY[x][y-1] != 1) amIAlone = true;
			if(y+1 < WORLD_HEIGHT)
				if(tilesXY[x][y+1] != 1) amIAlone = true;
			
			if(amIAlone || block.getType() == Block.GRASS_DISOLVE || block.getType() == Block.DIRT_DISSOLVE || block.getType() == Block.STONE_DISSOLVE || block.getType() == Block.ICE_DISSOLVE)
				collidableBlocks.add(block);
		}
		System.out.println("Collidable blocks count: " + collidableBlocks.size + " vs " + blocks.size);
		
		//Check the laser type
		for (Laser laser : lasers)
		{
			for(Block block: blocks)
			{
				if(overlapTest.pointInRectangle(block.getBounds(),laser.getPosition().x+0.5f,laser.getPosition().y-0.5f))
					laser.setType(Laser.GROUND);
				if(overlapTest.pointInRectangle(block.getBounds(),laser.getPosition().x+0.5f,laser.getPosition().y+Laser.SIZE+0.5f))
					laser.setType(Laser.CEILING);
				if(overlapTest.pointInRectangle(block.getBounds(),laser.getPosition().x-0.5f,laser.getPosition().y+0.5f))
					laser.setType(Laser.LEFT);
				if(overlapTest.pointInRectangle(block.getBounds(),laser.getPosition().x+Laser.SIZE+0.5f,laser.getPosition().y+0.5f))
					laser.setType(Laser.RIGHT);
			}
		}
		
		//calculate laser beam length
		for (Laser laser : lasers) {		
			float length = -1f;
			if(laser.getType() == Laser.GROUND)
			{
				for(Block block : blocks) {
					if(block.getPosition().x == laser.getPosition().x && block.getPosition().y > laser.getPosition().y) {
						if(length == -1)
							length = block.getPosition().y - laser.getPosition().y;		
						if(length > block.getPosition().y - laser.getPosition().y) 
							length = block.getPosition().y - laser.getPosition().y;						
					}
				}
				if(length == -1) length = 4f;				
				laser.setLength(length);
			}	
			if(laser.getType() == Laser.CEILING)
			{
				for(Block block : blocks) {
					if(block.getPosition().x == laser.getPosition().x && block.getPosition().y < laser.getPosition().y) {
						if(length == -1)
							length = laser.getPosition().y - block.getPosition().y;		
						if(length > laser.getPosition().y - block.getPosition().y) 
							length = laser.getPosition().y - block.getPosition().y;						
					}
				}
				if(length == -1) length = 4f;				
				laser.setLength(length);
			}	
			if(laser.getType() == Laser.LEFT)
			{
				for(Block block : blocks) {
					if(block.getPosition().y == laser.getPosition().y && block.getPosition().x > laser.getPosition().x) {
						if(length == -1)
							length = block.getPosition().x - laser.getPosition().x;		
						if(length > block.getPosition().x - laser.getPosition().x) 
							length = block.getPosition().x - laser.getPosition().x;						
					}
				}
				if(length == -1) length = 4f;				
				laser.setLength(length);
			}
			if(laser.getType() == Laser.RIGHT)
			{
				for(Block block : blocks) {
					if(block.getPosition().y == laser.getPosition().y && block.getPosition().x < laser.getPosition().x) {
						if(length == -1)
							length = laser.getPosition().x - block.getPosition().x;		
						if(length > laser.getPosition().x - block.getPosition().x) 
							length = laser.getPosition().x - block.getPosition().x;						
					}
				}
				if(length == -1) length = 4f;				
				laser.setLength(length);
			}
		}
		
		this.starEstimatedPoints = (this.coinCount * (this.countDownTimer - 200));
		
		this.setGameState(GAME_READY);

	}
	
	boolean match(int src, int dst) {
        return src == dst;
	}


	private void createDemoWorld() {
		bob = new Bob(new Vector2(0, 1));
		bob.setFacingLeft(false);

		for (int i = 0; i < 10; i++) {
			blocks.add(new Block(new Vector2(i, 0), Block.STONE));
			blocks.add(new Block(new Vector2(i, 6), Block.STONE));
			if (i > 2)
				blocks.add(new Block(new Vector2(i, 1), Block.STONE));
		}		

		blocks.add(new Block(new Vector2(9, 4), Block.STONE));
		blocks.add(new Block(new Vector2(9, 5), Block.STONE));

		blocks.add(new Block(new Vector2(6, 4), Block.STONE));
		blocks.add(new Block(new Vector2(6, 5), Block.STONE));
		
		
		blocks.add(new Block(new Vector2(10, 1), Block.GRASS));
		blocks.add(new Block(new Vector2(11, 1), Block.GRASS));
		blocks.add(new Block(new Vector2(12, 1), Block.GRASS));
		blocks.add(new Block(new Vector2(13, 1), Block.GRASS));
		blocks.add(new Block(new Vector2(14, 1), Block.GRASS));
		
		blocks.add(new Block(new Vector2(10, 0), Block.DIRT));
		blocks.add(new Block(new Vector2(11, 0), Block.DIRT));
		blocks.add(new Block(new Vector2(12, 0), Block.DIRT));
		blocks.add(new Block(new Vector2(13, 0), Block.DIRT));
		blocks.add(new Block(new Vector2(14, 0), Block.DIRT));
		
		blocks.add(new Block(new Vector2(14, 2), Block.SPIKES));
		
		//gosign.add(new gosign(new Vector2(4, 2)));
		
	}
	
	public static void setGameState(int state){
		game_state = state;
	}
	
	public int getGameState(){
		return this.game_state;
	}
	
	public int getLevelId()
	{
		return currentLevel;
	}

	public void reload() {
		// TODO Auto-generated method stub
		System.out.println("Reload pressed! ");		
		coinsCollected = 0;
		highScore = 0;
		countDownTimer = 500;
		
		for(Lift lift : lifts)
		{
			lift.resetPosition();
		}
		
		for(missile missile : missiles)
		{
			missile.resetStatus();
		}
		
		for(MovingSpike movingSpike : movingSpikes)
		{
			movingSpike.resetPosition();
		}
		
		for(Block block : blocks)
		{
			block.resetBlock();
		}
		
		for(Apple apple : apples)
		{
			apple.resetPosition();
		}
		/*blocks.clear();
		gosign.clear();
		coins.clear();
		lasers.clear();*/	
		
		//readWorldFromfile();
		//reactivate everything
		for(Coin coin : coins ){
			coin.active=true;
		}
		bob.setFacingLeft(false);
		bob.resetPosition();
		bob.setState(Bob.IDLE);
		this.setGameState(GAME_READY);
		
	}
}