package net.rastro.superdroid.model;

import net.rastro.superdroid.Assets;
import net.rastro.superdroid.controller.overlapTest;

import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;

public class missile {

	public static final float SIZE_W = 0.1f;
	public static final float SIZE_H = 0.3f;
	public static final float VELOCITY = 2.5f;
	
	public static final int STATE_FLYING = 1;	
	public static final int STATE_EXPLODING = 2;
	public static final int STATE_READY = 3;
	
	public static final float DETECTION_RANGE = 4f;
	
	private float stateTime = 0;

	Vector2 	MissilePosition = new Vector2();
	Vector2 	vel = new Vector2();
	Rectangle 	bounds = new Rectangle();
	boolean		active=true;	
	private Vector2		startPosition = new Vector2();
	int 		rocketState;
	private boolean didIHitBlock;
	private boolean	iHitBob;	

	public missile(Vector2 pos2, Vector2 startPos) {	
		this.MissilePosition.set(pos2.x+0.5f, pos2.y+0.5f);
		bounds.x = MissilePosition.x + SIZE_W;
    	bounds.y = MissilePosition.y + SIZE_H;
		rocketState=this.STATE_READY;
		this.startPosition.set(startPos);
		this.didIHitBlock = false;
		this.iHitBob = false;
			
	}
	
	public void update(float delta, Rectangle bobBounds, Vector2 bobPos)
	{
		if(World.game_state == World.GAME_LOST && this.rocketState == this.STATE_FLYING) return;
		if(World.game_state == World.GAME_FINISH && this.rocketState == this.STATE_FLYING) return; 		
		stateTime+=delta;
		if(this.rocketState == this.STATE_READY)
		{
			Rectangle rangeBoundaries = new Rectangle(startPosition.x-this.DETECTION_RANGE,
					startPosition.y-this.DETECTION_RANGE, 2*this.DETECTION_RANGE, 2*this.DETECTION_RANGE);			
			
			if(overlapTest.overlapRectangles(bobBounds, rangeBoundaries))
			{
				this.rocketState = this.STATE_FLYING;
			}
			else return;
		}
		if(this.didIHitBlock == true && this.rocketState == this.STATE_FLYING)
		{			
			this.rocketState = this.STATE_EXPLODING;
			Assets.playSound(Assets.explosionSound);
			stateTime = 0;		
			this.didIHitBlock = false;
		}
		if(this.rocketState == this.STATE_FLYING)
		{
			vel.set(bobPos);
        	vel.sub(MissilePosition).nor().mul(VELOCITY);
        	MissilePosition.add(vel.x * delta, vel.y * delta);
        	bounds.x = MissilePosition.x + SIZE_W;
        	bounds.y = MissilePosition.y + SIZE_H;
        	
        	if(overlapTest.overlapRectangles(bobBounds, bounds))
    		{
    			System.out.println("missile touched");
    			//this.active = false;
    			this.MissilePosition.set(bobPos);
    			this.rocketState = this.STATE_EXPLODING;
    			Assets.playSound(Assets.explosionSound);
    			stateTime = 0;		
    			this.iHitBob = true;
    		}
        	
		}
		else
		{
			if(this.stateTime>0.6 && this.rocketState != this.STATE_READY)
			{
				this.didIHitBlock = false;
				this.MissilePosition.set(startPosition.x+0.5f, startPosition.y+0.3f);
				this.rocketState = this.STATE_READY;	
				this.iHitBob = false;		
				bounds.x = MissilePosition.x + SIZE_W;
	        	bounds.y = MissilePosition.y + SIZE_H;
			}
		}
		
	}
	
	public boolean isActive()
	{
		return active;
	}
	
	public Vector2 getPosition() {
		return MissilePosition;
	}

	public Rectangle getBounds() {
		return bounds;
	}	
	public void setPosition(Vector2 pos2)
	{
		MissilePosition = pos2;
	}
	public void setActiveStatus(boolean b)
	{
		this.active = b;
	}
	public float getStateTime()
	{
		return this.stateTime;
	}
	public Vector2 getVel()
	{
		return this.vel;
	}
	public int getState()
	{
		return this.rocketState;
	}
	public void hitABlock()
	{
		this.didIHitBlock = true;
	}
	public boolean getHitABlock()
	{
		return this.didIHitBlock;
	}
	public boolean didIHitBob()
	{
		return this.iHitBob;
	}
	public void resetStatus()
	{
		this.rocketState = this.STATE_READY;
		this.MissilePosition.set(startPosition);
	}
	
}