package net.rastro.superdroid.screens;

import net.rastro.superdroid.SuperDroid;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;

public abstract class AbstractScreen implements Screen{
		    
    BitmapFont font;    
    SpriteBatch batch;
    protected final SuperDroid game;
    protected final  Stage stage;    
    private Skin skin;    
    float stateTime = 0f;
    
    public AbstractScreen(SuperDroid game )
        {
            this.game = game;
            stage = new Stage(900, 600, false);
        }
	
	 @Override
	    public void show()
	    {
	        // set the stage as the input processor
	        Gdx.input.setInputProcessor( stage );	        

	    }
	 
	 protected Skin getSkin()
	    {
	        if( skin == null ) {
	            FileHandle skinFile = Gdx.files.internal( "skin/uiskin.json" );
	            FileHandle textureFile = Gdx.files.internal( "skin/uiskin.png" );
	            skin = new Skin( skinFile, textureFile );
	        }
	        return skin;
	    }



	 
	 @Override
	    public void resize(
	        int width,
	        int height )
	    {
		// stage.setViewport(width, height, true);

	    }

	    @Override
	    public void render(
	        float delta )
	    {
	    	stateTime +=delta;
	        // (1) process the game logic

	        // update the actors
	        stage.act( delta );

	        // (2) draw the result

	        // clear the screen with the given RGB color (black)
	        Gdx.gl.glClearColor( 0f, 0f, 0f, 1f );
	        Gdx.gl.glClear( GL20.GL_COLOR_BUFFER_BIT );

	        // draw the actors
	        stage.draw();	     
	        //Table.drawDebug(stage);
	    }

	    @Override
	    public void hide()
	    {


	        // dispose the screen when leaving the screen;
	        // note that the dipose() method is not called automatically by the
	        // framework, so we must figure out when it's appropriate to call it
	        dispose();
	    }

	    @Override
	    public void pause()
	    {

	    }

	    @Override
	    public void resume()
	    {

	    }

	    @Override
	    public void dispose()
	    {	        

	        // as the collaborators are lazily loaded, they may be null

	        // the following call disposes the screen's stage but on my computer it
	        // crashes the game, so I've commented it out; more info can be found
	        // at: http://www.badlogicgames.com/forum/viewtopic.php?f=11&t=3624
	        // stage.dispose();

	        if( font != null ) font.dispose();
	        if( batch != null ) batch.dispose();
	    }

}
