package net.rastro.superdroid.screens;

import net.rastro.superdroid.Assets;
import net.rastro.superdroid.SuperDroid;
import net.rastro.superdroid.controller.WorldController;
import net.rastro.superdroid.model.World;
import net.rastro.superdroid.view.OnscreenControlRenderer;
import net.rastro.superdroid.view.WorldRenderer;

import com.badlogic.gdx.Application.ApplicationType;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.actions.FadeIn;
import com.badlogic.gdx.scenes.scene2d.actions.Sequence;
import com.badlogic.gdx.scenes.scene2d.ui.ClickListener;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.ImageButton;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.tablelayout.Table;
import com.badlogic.gdx.scenes.scene2d.ui.tablelayout.TableLayout;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Scaling;

public class GameScreen extends
AbstractScreen implements Screen, InputProcessor {
	
	private World 			world;
	private WorldRenderer 	renderer;
	private WorldController	controller;
	private OnscreenControlRenderer controlRenderer;
	
	private Table table;
	
	boolean reloadLevel=false;
	private boolean isGameOverScreenActive = false;
	private boolean isGameFinishedScreenActive = false;
	private boolean isPauseGameScreenActive = false;	

	
	private int width, height;	
	private int levelId;

	public GameScreen(SuperDroid game, int hashCode) {
		super(game);		
		
		Assets.loadCharacter();
		
		for(int i=0; i<16; i++)
		{
			System.out.println(game.btnHashCodes[i]);
			if(game.btnHashCodes[i] == hashCode)
			{
				levelId = i+1;
				break;
			}
		}
		// TODO Auto-generated constructor stub
	}

	

	@Override
	public void show() {
		world = new World(levelId);
		renderer = new WorldRenderer(world, false);
		controller = new WorldController(world);
		controlRenderer = new OnscreenControlRenderer(world);
		Gdx.input.setInputProcessor(this);
				
	}

	@Override
	public void render(float delta) {
		if(delta > 0.3) return;
		 // update the actors
        stage.act( delta );
		Gdx.gl.glClearColor(0.1f, 0.1f, 0.1f, 1);
		Gdx.gl.glClear(GL10.GL_COLOR_BUFFER_BIT);
			processKeys();
			controller.update(delta);
			renderer.render(delta);
			controlRenderer.render();		
				
			if(world.getGameState() == World.GAME_LOST && !isGameOverScreenActive)
			{
				showGameOverScreen();
				releaseAllKeys();
			}
			if(world.getGameState() == World.GAME_FINISH && !isGameFinishedScreenActive)
			{
				showGameFinishedScreen();
				releaseAllKeys();
			}
			if(world.getGameState() == World.GAME_PAUSED && !isPauseGameScreenActive)
			{
				showPauseGameScreen();
				releaseAllKeys();
			}
	        // draw the actors
			if(isGameOverScreenActive || isGameFinishedScreenActive || isPauseGameScreenActive)
			{				
				stage.draw();
				Table.drawDebug(stage);
			}
			if(world.getGameState() == World.GAME_READY || world.getGameState() == World.GAME_RUNNING)
				Gdx.input.setInputProcessor(this);
	}
	
	private void processKeys()
	{
		if (!Gdx.app.getType().equals(ApplicationType.Android)) return;
		float x0 = (Gdx.input.getX(0) / (float)Gdx.graphics.getWidth()) * width;
		float x1 = (Gdx.input.getX(1) / (float)Gdx.graphics.getWidth()) * width;
		float y0 = (Gdx.input.getY(0) / (float)Gdx.graphics.getHeight()) * height;
		float y1 = (Gdx.input.getY(1) / (float)Gdx.graphics.getHeight()) * height;

		boolean leftButton = (Gdx.input.isTouched(0) && x0 < 50 && y0 > this.height-150) || (Gdx.input.isTouched(1) && x1 < 50 && y1 > this.height-150);
		boolean rightButton = (Gdx.input.isTouched(0) && x0 > 90 && x0 < 134 && y0 > this.height-150) || (Gdx.input.isTouched(1) && x1 > 90 && x1 < 134 && y1 > this.height-150);
		boolean jumpButton = (Gdx.input.isTouched(0) && x0 > this.width-64 && x0 < this.width && y0 > this.height-64)
		|| (Gdx.input.isTouched(1) && x1 > this.width-64 && x1 < this.width && y1 > this.height-64);
		
		boolean upButton = (Gdx.input.isTouched(0) && y0 < this.height-50 && x0 <128 && y0 > this.height-150) || (Gdx.input.isTouched(1) && y1 < this.height-50 && x1 <128 && y1 > this.height-150);
		boolean downButton = (Gdx.input.isTouched(0) && y0 > this.height-90 && x0 < 128 && y0 > this.height-150) || (Gdx.input.isTouched(1) && y1 > this.height-90 && x1 < 128 && y1 > this.height-150);
		
		if(leftButton)
			controller.leftPressed();
		else
			controller.leftReleased();
		if(rightButton)
			controller.rightPressed();
		else
			controller.rightReleased();
		if(jumpButton)
			controller.jumpPressed();
		else
			controller.jumpReleased();
		if(upButton)
			controller.upPressed();
		else
			controller.upReleased();
		if(downButton)
			controller.downPressed();
		else
			controller.downReleased();
	}
	
	private void showGameOverScreen()
	{
		Gdx.input.setInputProcessor(stage);
		Skin skin = super.getSkin();
		//stage.addActor(img);
		NinePatch sPatch = new NinePatch(new Texture(Gdx.files.internal("images/menuskin.png")), 8, 8, 8, 8);		
        table = new Table( skin );
        //table.width = stage.width();
        //table.height = stage.height();
        table.width=250;
        table.height=160;
        
        table.x = (stage.width()/2)-table.width/2;
        table.y = (stage.height()/2)-table.height/2;       
        table.color.a = 0f;               
        table.setBackground(sPatch);
        Sequence actions = Sequence.$( FadeIn.$( 0.50f ));
        table.action( actions );                
        
        // retrieve the table's layout
        TableLayout layout = table.getTableLayout();
        
        table.add( "Oh No! Game Over!" ).padBottom(20).colspan(3);
        table.row();
        table.add( "Better luck next time!" ).padBottom(20).colspan(3);
        table.row();
        
        Texture menubtnImgUp = new Texture(Gdx.files.internal("images/menu_button.png"));
        Texture menubtnImgDown = new Texture(Gdx.files.internal("images/menu_button_press.png"));
    	ImageButton menuBtn = new ImageButton(new TextureRegion(menubtnImgUp, 0, 0, 48, 48), new TextureRegion(menubtnImgDown, 0, 0, 48, 48));
    	menuBtn.width(48);
    	menuBtn.setClickListener( new ClickListener() {
            @Override
            public void click(
                Actor actor,
                float x,
                float y )
            {
            	System.out.println("Exit button clicked");
                //game.getSoundManager().play( TyrianSound.CLICK );
            	Assets.playSound(Assets.clickSound);
            	 game.setScreen(new MainMenu(game));
    			table.color.a = 0f;
            }
        } );
    	layout.add( menuBtn ).width(48);
    	
    	Texture restartbtnImgUp = new Texture(Gdx.files.internal("images/restart_btn.png"));
        Texture restartbtnImgDown = new Texture(Gdx.files.internal("images/restart_btn_press.png"));
    	ImageButton restartLevelBtn = new ImageButton(new TextureRegion(restartbtnImgUp, 0, 0, 48, 48), new TextureRegion(restartbtnImgDown, 0, 0, 48, 48));
    	restartLevelBtn.width(48);
    	restartLevelBtn.setClickListener( new ClickListener() {
            @Override
            public void click(
                Actor actor,
                float x,
                float y )
            {
            	System.out.println("Restart button clicked");
                //game.getSoundManager().play( TyrianSound.CLICK );
            	Assets.playSound(Assets.clickSound);
            	renderer.cam.position.set(renderer.CAMERA_WIDTH / 2f, renderer.CAMERA_HEIGHT / 2f, 0);
    			world.reload();
    			reloadLevel = false;
    			isGameOverScreenActive = false;
    			world.setGameState(World.GAME_READY);    
    			table.color.a = 0f;
            }
        } );
    	layout.add( restartLevelBtn ).width(48);   
        layout.center();
      //  layout.padRight(20).padBottom(20);        
        isGameOverScreenActive = true;        
        stage.addActor( table );
	}
	
	private void showGameFinishedScreen()
	{
		Gdx.input.setInputProcessor(stage);
		Skin skin = super.getSkin();
		Assets.playSound(Assets.finishSound);
		
		if(SuperDroid.settings.levelsFinished<levelId)
			SuperDroid.settings.levelsFinished = levelId;
		
		//stage.addActor(img);	
		
		
		Array<Image> starImg = new Array<Image>();
		if(world.starEstimatedPoints <= world.highScore && world.coinsCollected == world.coinCount)
		{
			starImg.add( new Image(Assets.starBigOn, Scaling.stretch ) );
			starImg.add( new Image(Assets.starBigOn, Scaling.stretch ) );
			starImg.add( new Image(Assets.starBigOn, Scaling.stretch ) );
			if(SuperDroid.settings.levelScores[levelId-1] < world.highScore)
				SuperDroid.settings.levelStars[levelId-1] = 3;
		}
		else if(world.starEstimatedPoints/2 <= world.highScore)
		{
			starImg.add( new Image(Assets.starBigOn, Scaling.stretch ) );
			starImg.add( new Image(Assets.starBigOn, Scaling.stretch ) );
			starImg.add( new Image(Assets.starBigOff, Scaling.stretch ) );
			if(SuperDroid.settings.levelScores[levelId-1] < world.highScore)
				SuperDroid.settings.levelStars[levelId-1] = 2;
		} else {
			starImg.add( new Image(Assets.starBigOn, Scaling.stretch ) );
			starImg.add( new Image(Assets.starBigOff, Scaling.stretch ) );
			starImg.add( new Image(Assets.starBigOff, Scaling.stretch ) );
			if(SuperDroid.settings.levelScores[levelId-1] < world.highScore)
				SuperDroid.settings.levelStars[levelId-1] = 1;
		}
						
		
		NinePatch sPatch = new NinePatch(new Texture(Gdx.files.internal("images/menuskin.png")), 8, 8, 8, 8);		
        table = new Table( skin );
        //table.width = stage.width();
        //table.height = stage.height();
        table.width=250;
        table.height=300;
        
        table.x = (stage.width()/2)-table.width/2;
        table.y = (stage.height()/2)-table.height/2;       
        table.color.a = 0f;               
        table.setBackground(sPatch);
        Sequence actions = Sequence.$( FadeIn.$( 0.50f ));
        table.action( actions );                               
        
        table.add( "LEVEL COMPLETE!" ).padBottom(20).colspan(3);
        table.row();
        if(SuperDroid.settings.levelScores[levelId-1] < world.highScore)
        {
			SuperDroid.settings.levelScores[levelId-1] = world.highScore;
			table.add( new Image(Assets.starSmallOn, Scaling.stretch ) ).padBottom(5).center().colspan(3);
			table.row();
			table.add( "NEW HIGHSCORE!" ).colspan(3).padBottom(20);			        	
        }
        else
        {
        	table.add( "Nice Score!" ).colspan(3).padBottom(20);
        }
        table.row();
        
        String score = "Score: " + world.highScore;
        table.add( score ).padBottom(10).colspan(3);
        table.row();
        
     // retrieve the table's layout
        TableLayout layout = table.getTableLayout();
        
        layout.add( starImg.get(0) ).width(40).padBottom(20);
        layout.add( starImg.get(1) ).width(40).padBottom(20);
        layout.add( starImg.get(2) ).width(40).padBottom(20);
        layout.row();
            	                 
        
        Texture menubtnImgUp = new Texture(Gdx.files.internal("images/menu_button.png"));
        Texture menubtnImgDown = new Texture(Gdx.files.internal("images/menu_button_press.png"));
    	ImageButton menuBtn = new ImageButton(new TextureRegion(menubtnImgUp, 0, 0, 48, 48), new TextureRegion(menubtnImgDown, 0, 0, 48, 48));
    	menuBtn.width(48);
    	menuBtn.setClickListener( new ClickListener() {
            @Override
            public void click(
                Actor actor,
                float x,
                float y )
            {
            	System.out.println("Exit button clicked");
                //game.getSoundManager().play( TyrianSound.CLICK );
            	Assets.playSound(Assets.clickSound);
            	 game.setScreen(new MainMenu(game));
    			table.color.a = 0f;
            }
        } );
    	layout.add( menuBtn ).width(48);
    	
    	Texture restartbtnImgUp = new Texture(Gdx.files.internal("images/restart_btn.png"));
        Texture restartbtnImgDown = new Texture(Gdx.files.internal("images/restart_btn_press.png"));
    	ImageButton restartLevelBtn = new ImageButton(new TextureRegion(restartbtnImgUp, 0, 0, 48, 48), new TextureRegion(restartbtnImgDown, 0, 0, 48, 48));
    	restartLevelBtn.width(48);
    	restartLevelBtn.setClickListener( new ClickListener() {
            @Override
            public void click(
                Actor actor,
                float x,
                float y )
            {
            	System.out.println("Restart button clicked");
                //game.getSoundManager().play( TyrianSound.CLICK );
            	Assets.playSound(Assets.clickSound);
            	renderer.cam.position.set(renderer.CAMERA_WIDTH / 2f, renderer.CAMERA_HEIGHT / 2f, 0);
    			world.reload();
    			reloadLevel = false;
    			isGameFinishedScreenActive = false;
    			world.setGameState(World.GAME_READY);    
    			table.color.a = 0f;
            }
        } );
    	layout.add( restartLevelBtn ).width(48);   
    	
        Texture btnImgUp = new Texture(Gdx.files.internal("images/nextBtn.png"));
        Texture btnImgDown = new Texture(Gdx.files.internal("images/nextBtnPress.png"));
    	ImageButton nextLevelBtn = new ImageButton(new TextureRegion(btnImgUp, 0, 0, 48, 48), new TextureRegion(btnImgDown, 0, 0, 48, 48));
    	nextLevelBtn.setClickListener( new ClickListener() {
            @Override
            public void click(
                Actor actor,
                float x,
                float y )
            {
            	System.out.println("Next level button clicked");
                //game.getSoundManager().play( TyrianSound.CLICK );
            	Assets.playSound(Assets.clickSound);
            	game.setScreen(new GameScreen(game, game.btnHashCodes[levelId]));
            }
        } );
    	nextLevelBtn.width(48);
    	layout.add( nextLevelBtn ).width(48);    	
        layout.row();
        
        layout.center();
      //  layout.padRight(20).padBottom(20);        
        isGameFinishedScreenActive = true;        
        stage.addActor( table );
        
        //Lets save the results
        SuperDroid.settingsManager.save();
	}	
	
	private void showPauseGameScreen()
	{
		Gdx.input.setInputProcessor(stage);
		Skin skin = super.getSkin();
		//stage.addActor(img);
		NinePatch sPatch = new NinePatch(new Texture(Gdx.files.internal("images/menuskin.png")), 8, 8, 8, 8);		
        table = new Table( skin );
        //table.width = stage.width();
        //table.height = stage.height();
        table.width=250;
        table.height=120;
        
        table.x = (stage.width()/2)-table.width/2;
        table.y = (stage.height()/2)-table.height/2;       
        table.color.a = 0f;               
        table.setBackground(sPatch);
        Sequence actions = Sequence.$( FadeIn.$( 0.50f ));
        table.action( actions );                               
        
        table.add( "GAME PAUSED!" ).padBottom(20).colspan(3);
        table.row();        
        
     // retrieve the table's layout
        TableLayout layout = table.getTableLayout();
            	                 
        
        Texture menubtnImgUp = new Texture(Gdx.files.internal("images/menu_button.png"));
        Texture menubtnImgDown = new Texture(Gdx.files.internal("images/menu_button_press.png"));
    	ImageButton menuBtn = new ImageButton(new TextureRegion(menubtnImgUp, 0, 0, 48, 48), new TextureRegion(menubtnImgDown, 0, 0, 48, 48));
    	menuBtn.width(48);
    	menuBtn.setClickListener( new ClickListener() {
            @Override
            public void click(
                Actor actor,
                float x,
                float y )
            {
            	System.out.println("Exit button clicked");
                //game.getSoundManager().play( TyrianSound.CLICK );
            	Assets.playSound(Assets.clickSound);
            	 game.setScreen(new MainMenu(game));
    			table.color.a = 0f;
            }
        } );
    	layout.add( menuBtn ).width(48);
    	
    	Texture restartbtnImgUp = new Texture(Gdx.files.internal("images/restart_btn.png"));
        Texture restartbtnImgDown = new Texture(Gdx.files.internal("images/restart_btn_press.png"));
    	ImageButton restartLevelBtn = new ImageButton(new TextureRegion(restartbtnImgUp, 0, 0, 48, 48), new TextureRegion(restartbtnImgDown, 0, 0, 48, 48));
    	restartLevelBtn.width(48);
    	restartLevelBtn.setClickListener( new ClickListener() {
            @Override
            public void click(
                Actor actor,
                float x,
                float y )
            {
            	System.out.println("Restart button clicked");
                //game.getSoundManager().play( TyrianSound.CLICK );
            	Assets.playSound(Assets.clickSound);
            	renderer.cam.position.set(renderer.CAMERA_WIDTH / 2f, renderer.CAMERA_HEIGHT / 2f, 0);
    			world.reload();
    			reloadLevel = false;
    			isPauseGameScreenActive = false;
    			world.setGameState(World.GAME_READY);    
    			table.color.a = 0f;
            }
        } );
    	layout.add( restartLevelBtn ).width(48);   
    	
        Texture btnImgUp = new Texture(Gdx.files.internal("images/nextBtn.png"));
        Texture btnImgDown = new Texture(Gdx.files.internal("images/nextBtnPress.png"));
    	ImageButton continueLevelBtn = new ImageButton(new TextureRegion(btnImgUp, 0, 0, 48, 48), new TextureRegion(btnImgDown, 0, 0, 48, 48));
    	continueLevelBtn.setClickListener( new ClickListener() {
            @Override
            public void click(
                Actor actor,
                float x,
                float y )
            {
            	System.out.println("Continue button clicked");
                //game.getSoundManager().play( TyrianSound.CLICK );
            	Assets.playSound(Assets.clickSound);     	    			
    			isPauseGameScreenActive = false;
    			world.setGameState(World.GAME_RUNNING);    
    			table.color.a = 0f;
            }
        } );
    	continueLevelBtn.width(48);
    	layout.add( continueLevelBtn ).width(48);    	
        layout.row();
        
        layout.center();
      //  layout.padRight(20).padBottom(20);        
        isPauseGameScreenActive = true;        
        stage.addActor( table );
	}	

	@Override
	public void resize(int width, int height) {
		renderer.setSize(width, height);
		this.width = width;
		this.height = height;
		controlRenderer.setwwHH(width, height);
	}

	@Override
	public void hide() {
		Gdx.input.setInputProcessor(null);
	}

	@Override
	public void pause() {
		// TODO Auto-generated method stub
	}

	@Override
	public void resume() {
		// TODO Auto-generated method stub
	}

	@Override
	public void dispose() {
		Gdx.input.setInputProcessor(null);
	}

	// * InputProcessor methods ***************************//

	@Override
	public boolean keyDown(int keycode) {
		if(world.getGameState() == World.GAME_READY)
			world.setGameState(World.GAME_RUNNING);
		if (keycode == Keys.LEFT)
			controller.leftPressed();
		if (keycode == Keys.RIGHT)
			controller.rightPressed();
		if (keycode == Keys.SPACE)
			controller.jumpPressed();
		if (keycode == Keys.X)
			controller.firePressed();
		if (keycode == Keys.UP)
			controller.upPressed();
		if (keycode == Keys.DOWN)
			controller.downPressed();
		return true;
	}
	
	private void releaseAllKeys()
	{		
			controller.leftReleased();
			controller.rightReleased();
			controller.jumpReleased();
			controller.fireReleased();	
			controller.upReleased();
			controller.downReleased();
	}

	@Override
	public boolean keyUp(int keycode) {
		if (keycode == Keys.LEFT)
			controller.leftReleased();
		if (keycode == Keys.RIGHT)
			controller.rightReleased();
		if (keycode == Keys.SPACE)
			controller.jumpReleased();
		if (keycode == Keys.X)
			controller.fireReleased();	
		if (keycode == Keys.UP)
			controller.upReleased();
		if (keycode == Keys.DOWN)
			controller.downReleased();
		if(keycode == Keys.ESCAPE || keycode == Keys.P)
			world.setGameState(World.GAME_PAUSED);
			
		return true;
	}

	@Override
	public boolean keyTyped(char character) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean touchDown(int x, int y, int pointer, int button) {
		if(world.getGameState() == World.GAME_READY)
			world.setGameState(World.GAME_RUNNING);
		/*
		if (!Gdx.app.getType().equals(ApplicationType.Android))
			return false;
		int zaebal = this.height-y;
		System.out.println(x + "," + zaebal);
		
		if(y < this.height-50 && x <128)       	 
				controller.upPressed();
        if(y > this.height-90 && x < 128)
        		controller.downPressed();
       	 
		
		if(x < 50)
			controller.leftPressed();
        if(x > 90 && x < 134)
        	controller.rightPressed();
        if(x > this.width-64 && x < this.width && y > this.height-64)
        	controller.jumpPressed();*/

		return true;
	}
	
	private Vector3 touchPoint = new Vector3();
	
	@Override
	public boolean touchUp(int x, int y, int pointer, int button) {
		
	        // touchPoint.set(x, height-y, 0);
		//Game pause button pressed?
		if(x < 50 && y < 58)
		{
			Assets.playSound(Assets.clickSound);
			world.setGameState(World.GAME_PAUSED);
		}
		
		if(x > 68 && x < 116 && y < 58)
		{
			SuperDroid.settings.soundEnabled = !SuperDroid.settings.soundEnabled;
			Assets.playSound(Assets.clickSound);
			if(SuperDroid.settings.soundEnabled)
				Assets.music.play();
			else
				Assets.music.stop();
		}
		
		/*if (!Gdx.app.getType().equals(ApplicationType.Android))
			return false;
		
		
         if(y < this.height-50 && x <128 && y > this.height-150)        	 
         		controller.upReleased();
         if(y > this.height-90 && x < 128 && y > this.height-150)
        	 	controller.downReleased();

		
		if(x < 50 && y > this.height-150)
			controller.leftReleased();
        if(x > 90 && x < 134 && y > this.height-150)
        	controller.rightReleased();
        if(x > this.width-64 && x < this.width && y > this.height-64)
        	controller.jumpReleased();
        */
		return true;
	}

	@Override
	public boolean touchDragged(int x, int y, int pointer) {
		/*if (!Gdx.app.getType().equals(ApplicationType.Android))
			return false;
		
		// First release all buttons
		if(x > this.width/2)
			controller.jumpReleased();
		if(x < this.width/2)
		{
			controller.rightReleased();
			controller.leftReleased();
			controller.downReleased();
			controller.upReleased();
		}
		
		//Now check which buttons should still be active
		if(y < this.height-50 && x <128 && y > this.height-150)       	 
			controller.upPressed();
    if(y > this.height-90 && x < 128 && y > this.height-150)
    		controller.downPressed();
   	 
	
	if(x < 50 && y > this.height-150)
		controller.leftPressed();
    if(x > 90 && x < 134 && y > this.height-150)
    	controller.rightPressed();
    if(x > this.width-64 && x < this.width && y > this.height-64)
    	controller.jumpPressed();
		*/
		return true;
	}

	public boolean touchMoved(int x, int y) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean scrolled(int amount) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean mouseMoved(int x, int y) {
		// TODO Auto-generated method stub
		return false;
	}
}