package net.rastro.superdroid.screens;

import net.rastro.superdroid.Assets;
import net.rastro.superdroid.SuperDroid;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.actions.FadeIn;
import com.badlogic.gdx.scenes.scene2d.actions.Sequence;
import com.badlogic.gdx.scenes.scene2d.ui.ClickListener;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton.TextButtonStyle;
import com.badlogic.gdx.scenes.scene2d.ui.tablelayout.Table;
import com.badlogic.gdx.scenes.scene2d.ui.tablelayout.TableLayout;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Scaling;


public class LevelSelectScreen extends
AbstractScreen
 {


	private Texture texture;
    private Image img;
    private Table table;
    private Table tableButtons;
    private Image logoImage;
    
    
    
	public LevelSelectScreen(
	        SuperDroid game )
	    {
	        super( game );
	    }




	@Override
	public void show() {
		// TODO Auto-generated method stub				
		Skin skin = super.getSkin();
		Gdx.input.setInputProcessor(stage);
		
        texture = new Texture(Gdx.files.internal("images/background.jpg"));
        // set the linear texture filter to improve the image stretching
       // texture.setFilter( TextureFilter.Linear, TextureFilter.Linear );        


        font = new BitmapFont(Gdx.files.internal("fonts/font.fnt"), false);          
                
          //ui = new Stage(width, height, true);
          img = new Image(new TextureRegion(texture), Scaling.stretch );
          img.x = 0;
          img.y = 0;
          img.width = stage.width();
          img.height = stage.height();   
          
       // this is needed for the fade-in effect to work correctly; we're just
          // making the image completely transparent          

          // configure the fade-in/out effect on the splash image
                   
          //stage.addActor(img);
          table = new Table( skin );
          table.width = stage.width();
          table.height = stage.height();
          tableButtons = new Table( skin );
          tableButtons.width = stage.width();
          tableButtons.height = stage.height();
          
          table.color.a = 0f;
          tableButtons.color.a = 0f;
          img.color.a = 0f;
          
          logoImage = new Image(Assets.gameLogo, Scaling.stretch);
          logoImage.y = stage.height()-logoImage.height-10f;
          logoImage.x = 10f;
          logoImage.color.a = 0f;
          Sequence logoFadeIn = Sequence.$( FadeIn.$( 0.75f ));
          logoImage.action( logoFadeIn );
          
          
          Sequence actionIMG = Sequence.$( FadeIn.$( 0.75f ));
          Sequence actionTable = Sequence.$( FadeIn.$( 0.75f ));
          Sequence actionTableButtons = Sequence.$( FadeIn.$( 0.75f ));
          table.action( actionTable );
          tableButtons.action( actionTableButtons );  
          img.action( actionIMG );                    
          
          stage.addActor(img);          
          stage.addActor( table );
          stage.addActor(tableButtons);
          stage.addActor( logoImage );
          
          TableLayout btnLayout = tableButtons.getTableLayout();
          
          btnLayout.row();
          
          TextButton backToMenuBtn = new TextButton( "Back To Main Menu", skin );
          backToMenuBtn.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("back clicked");
            	  Assets.playSound(Assets.clickSound);
                  game.setScreen( new MainMenu(game) );                 
              }
          } ); 
      	  btnLayout.add( backToMenuBtn ).width(180);
      	  
      	  btnLayout.bottom().left();
          
          // retrieve the table's layout
          TableLayout layout = table.getTableLayout();                   
          
          Array<TextButton> levelBtns = new Array<TextButton>();
          Array<Image> starImg = new Array<Image>();
          
          layout.row();
          for(int i=0; i<=16; i++)
          {        	  
        	  if(i % 4 == 0 && i!=0)
        	  {
        		  layout.row();
        		  for(int k=i-4; k<i; k++)
        		  {
        			  if(SuperDroid.settings.levelStars[k] == 0)
        			  {
        				starImg.add( new Image(Assets.starSmallOff, Scaling.stretch ) );
        				starImg.add( new Image(Assets.starSmallOff, Scaling.stretch ) );
        				starImg.add( new Image(Assets.starSmallOff, Scaling.stretch ) );
        			  }
        			  else
        			  	starImg.add( new Image(Assets.starSmallOn, Scaling.stretch ) );
        			  
        			  if(SuperDroid.settings.levelStars[k] == 3)
        			  {        				  
        				  starImg.add( new Image(Assets.starSmallOn, Scaling.stretch ) );
        				  starImg.add( new Image(Assets.starSmallOn, Scaling.stretch ) );        				  
        			  }
        			  if(SuperDroid.settings.levelStars[k] == 2)
        			  {        				
        				  starImg.add( new Image(Assets.starSmallOn, Scaling.stretch ) );
        				  starImg.add( new Image(Assets.starSmallOff, Scaling.stretch ) );        				  
        			  }
        			  if(SuperDroid.settings.levelStars[k] == 1)
        			  {        				
        				  starImg.add( new Image(Assets.starSmallOff, Scaling.stretch ) );
        				  starImg.add( new Image(Assets.starSmallOff, Scaling.stretch ) );        				  
        			  }
        			  int index = 3*k;
        			  layout.add(starImg.get(0+index)).width(20).height(20).padLeft(25).padBottom(15);
    				  layout.add(starImg.get(1+index)).width(20).height(20).padBottom(15);
    				  layout.add(starImg.get(2+index)).width(20).height(20).padRight(25).padBottom(15);
        		  }        		  
        			  layout.row();
        	  }
        	  
        	  if(i==16) break;
        	  
        	  if(SuperDroid.settings.levelsFinished < i)        		  
        		  levelBtns.add(new TextButton( Integer.toString(i+1) , skin.getStyle("inactive", TextButtonStyle.class) ));
        	  else
        		  levelBtns.add(new TextButton( Integer.toString(i+1) , skin ));
        	  layout.add(levelBtns.get(i)).width(60).height(60).padBottom(3).colspan(3);  
        	  
        	  //Well Im just guessing the next line, it looks like WTF?
        	  game.btnHashCodes[i] = levelBtns.get(i).hashCode();
        	 // System.out.println(btnHashCodes[i] + " == ");
        	  if(SuperDroid.settings.levelsFinished < i) continue; 
        	  levelBtns.get(i).setClickListener( new ClickListener() {
                  @Override
                  public void click(
                      Actor actor,
                      float x,
                      float y )
                  {                	  
                	  Assets.playSound(Assets.clickSound);
                	  //System.out.println(actor.hashCode());
                	  Assets.menu.stop();
                	  Assets.playMusic(Assets.music);
                      game.setScreen( new GameScreen(game, actor.hashCode()) );                 
                  }
              } );
          }
          
       // register the button "start game"
         /* TextButton startGameButton = new TextButton( "1", skin );
          startGameButton.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("Start Game clicked");
            	  Assets.playSound(Assets.clickSound);
                  game.setScreen( new GameScreen(game) );                 
              }
          } );
          layout.add(startGameButton).width(60).height(60).pad(10);
          
          TextButton aboutGameButton = new TextButton( "2", skin );
          aboutGameButton.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("About clicked");
            	  Assets.playSound(Assets.clickSound);
                 // game.setScreen( new GameScreen(game) );                 
              }
          } );       
          layout.add(aboutGameButton).width(60).height(60).pad(10);
          
          TextButton exitGameButton = new TextButton( "3", skin );
          exitGameButton.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("Exit Game clicked");
            	  Assets.playSound(Assets.clickSound);
                  Gdx.app.exit();               
              }
          } );

          layout.add(exitGameButton).width(60).height(60).pad(10);*/
          layout.center();
		
	}   
}