package net.rastro.superdroid.screens;

import net.rastro.superdroid.Assets;
import net.rastro.superdroid.SuperDroid;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.actions.FadeIn;
import com.badlogic.gdx.scenes.scene2d.actions.Sequence;
import com.badlogic.gdx.scenes.scene2d.ui.ClickListener;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.tablelayout.Table;
import com.badlogic.gdx.scenes.scene2d.ui.tablelayout.TableLayout;
import com.badlogic.gdx.utils.Scaling;


public class MainMenu extends
AbstractScreen
 {


	private Texture texture;
    private Image img;
    private Table table;
    private Image logoImage;
    private Table tableMorePlatforms;
    private Texture platformTexture; 
    
    
    
	public MainMenu(
	        SuperDroid game )
	    {
	        super( game );
	    }




	@Override
	public void show() {
		if(!Assets.menu.isPlaying())
		{
			Assets.music.stop();
			Assets.creditsMusic.stop();
			Assets.playMusic(Assets.menu);
		}
		
		Skin skin = super.getSkin();
		Gdx.input.setInputProcessor(stage);
		
        texture = new Texture(Gdx.files.internal("images/background.jpg"));
        platformTexture  = new Texture(Gdx.files.internal("images/platforms.png"));
        // set the linear texture filter to improve the image stretching
       // texture.setFilter( TextureFilter.Linear, TextureFilter.Linear );        


        font = new BitmapFont(Gdx.files.internal("fonts/font.fnt"), false);          
                
          //ui = new Stage(width, height, true);
          img = new Image(new TextureRegion(texture), Scaling.stretch );
          img.x = 0;
          img.y = 0;
          img.width = stage.width();
          img.height = stage.height();   
          
          logoImage = new Image(Assets.gameLogo, Scaling.stretch);
          
       // this is needed for the fade-in effect to work correctly; we're just
          // making the image completely transparent          

          // configure the fade-in/out effect on the splash image
                   
          //stage.addActor(img);
          table = new Table( skin );
          table.width = stage.width();
          table.height = stage.height();
          
          tableMorePlatforms = new Table(skin);
          tableMorePlatforms.color.a = 0f;
          tableMorePlatforms.width = stage.width();
          tableMorePlatforms.height = stage.height();
          
          table.color.a = 0f;
          img.color.a = 0f;
          
          
          
          Sequence actionIMG = Sequence.$( FadeIn.$( 0.50f ));
          Sequence actionTable = Sequence.$( FadeIn.$( 0.50f ));
          Sequence actionTablePlatforms = Sequence.$( FadeIn.$( 0.50f ));
          table.action( actionTable );  
          tableMorePlatforms.action( actionTablePlatforms );
          img.action( actionIMG );                    
                    
          stage.addActor(img);          
          stage.addActor( table );
          stage.addActor(tableMorePlatforms);
          
          
          tableMorePlatforms.row();
          tableMorePlatforms.add("Available for ");
          tableMorePlatforms.add(new Image(new TextureRegion(platformTexture,0,0,253,75), Scaling.stretch ));
          tableMorePlatforms.add(" at http://morf.lv");
          tableMorePlatforms.bottom().center().pad(10);
          
          
          // retrieve the table's layout
          TableLayout layout = table.getTableLayout();
          
          layout.add(logoImage).padBottom(15);
          layout.row();
          
       // register the button "start game"
          TextButton startGameButton = new TextButton( "Start game", skin );
          startGameButton.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("Start Game clicked");
            	  Assets.playSound(Assets.clickSound);
                  //game.setScreen( new GameScreen(game) );
            	  game.setScreen( new LevelSelectScreen(game) );
              }
          } );
          layout.add(startGameButton).padLeft(10).width(200);
          layout.row();
          
          TextButton helpButton = new TextButton( "Help", skin );
          helpButton.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("Help clicked");
            	  Assets.playSound(Assets.clickSound);
                  //game.setScreen( new OptionsScreen(game) );                 
              }
          } );       
          layout.add(helpButton).padLeft(10).padTop(10).width(200);
          layout.row();
          
          TextButton optionsButton = new TextButton( "Options", skin );
          optionsButton.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("About clicked");
            	  Assets.playSound(Assets.clickSound);
                  game.setScreen( new OptionsScreen(game) );          
              }
          } );       
          layout.add(optionsButton).padLeft(10).padTop(10).width(200);
          layout.row();
          
          TextButton aboutGameButton = new TextButton( "About", skin );
          aboutGameButton.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("About clicked");
            	  Assets.playSound(Assets.clickSound);
                  game.setScreen( new CreditScreen(game) );                 
              }
          } );       
          layout.add(aboutGameButton).padLeft(10).padTop(10).width(200);
          layout.row();
          
          TextButton exitGameButton = new TextButton( "Exit", skin );
          exitGameButton.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("Exit Game clicked");
            	  Assets.playSound(Assets.clickSound);
                  Gdx.app.exit();               
              }
          } );

          layout.add(exitGameButton).padLeft(10).padTop(10).width(200);
          layout.center();
		
	}   
}