package net.rastro.superdroid.screens;

import net.rastro.superdroid.Assets;
import net.rastro.superdroid.SuperDroid;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.actions.FadeIn;
import com.badlogic.gdx.scenes.scene2d.actions.Sequence;
import com.badlogic.gdx.scenes.scene2d.ui.CheckBox;
import com.badlogic.gdx.scenes.scene2d.ui.ClickListener;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Slider;
import com.badlogic.gdx.scenes.scene2d.ui.Slider.ValueChangedListener;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.tablelayout.Table;
import com.badlogic.gdx.scenes.scene2d.ui.tablelayout.TableLayout;
import com.badlogic.gdx.utils.Scaling;


public class OptionsScreen extends
AbstractScreen
 {


	private Texture texture;
    private Image img;
    private Table table;
    private Table tableButtons;
    private Image logoImage;
    private Image characterImage;
    
    
    
	public OptionsScreen(
	        SuperDroid game )
	    {
	        super( game );
	    }




	@Override
	public void show() {
		// TODO Auto-generated method stub				
		Skin skin = super.getSkin();
		Gdx.input.setInputProcessor(stage);
		
        texture = new Texture(Gdx.files.internal("images/background.jpg"));
        // set the linear texture filter to improve the image stretching
       // texture.setFilter( TextureFilter.Linear, TextureFilter.Linear );  
        


        font = new BitmapFont(Gdx.files.internal("fonts/font.fnt"), false);          
                
          //ui = new Stage(width, height, true);
          img = new Image(new TextureRegion(texture), Scaling.stretch );
          img.x = 0;
          img.y = 0;
          img.width = stage.width();
          img.height = stage.height();   
          
          logoImage = new Image(Assets.gameLogo, Scaling.stretch);
          logoImage.y = stage.height()-logoImage.height-10f;
          logoImage.x = 10f;
          logoImage.color.a = 0f;
          Sequence logoFadeIn = Sequence.$( FadeIn.$( 0.75f ));
          logoImage.action( logoFadeIn );
          
       // this is needed for the fade-in effect to work correctly; we're just
          // making the image completely transparent          

          // configure the fade-in/out effect on the splash image
                   
          //stage.addActor(img);
          table = new Table( skin );
          table.width = stage.width();
          table.height = stage.height();
          
          tableButtons = new Table( skin );
          tableButtons.width = stage.width();
          tableButtons.height = stage.height();
          
          table.color.a = 0f;
          tableButtons.color.a = 0f;
          img.color.a = 0f;
          
          
          
          Sequence actionIMG = Sequence.$( FadeIn.$( 0.50f ));
          Sequence actionTable = Sequence.$( FadeIn.$( 0.50f ));
          Sequence actionTableButtons = Sequence.$( FadeIn.$( 0.75f ));          
          table.action( actionTable );  
          img.action( actionIMG );           
          tableButtons.action( actionTableButtons );          
          
          stage.addActor(img);          
          stage.addActor( table );
          stage.addActor(tableButtons);
          stage.addActor(logoImage);

          TableLayout btnLayout = tableButtons.getTableLayout(); 
                       
          btnLayout.row();
          
          TextButton backToMenuBtn = new TextButton( "Back To Main Menu", skin );
          backToMenuBtn.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  System.out.println("back clicked");
            	  Assets.playSound(Assets.clickSound);
            	  SuperDroid.settingsManager.save();
                  game.setScreen( new MainMenu(game) );                 
              }
          } ); 
      	  btnLayout.add( backToMenuBtn ).width(180);
      	        	  
      	  
      	  btnLayout.bottom().left();
          
          // retrieve the table's layout
          table.add( "OPTIONS" ).padBottom(20).colspan(3).left();
          table.row();
          
          TableLayout layout = table.getTableLayout();                    
                    
       // create the labels widgets
          final CheckBox soundEffectsCheckbox = new CheckBox( skin );
          soundEffectsCheckbox.setChecked( SuperDroid.settings.soundEnabled );
          soundEffectsCheckbox.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
                  boolean enabled = soundEffectsCheckbox.isChecked();
                  SuperDroid.settings.soundEnabled = enabled;
                  Assets.playSound(Assets.clickSound);                  
              }
          } );
          layout.add(soundEffectsCheckbox );
          table.add(" Sound effects ").left().padLeft(10);
          layout.row();
          
          final CheckBox musicCheckBox = new CheckBox( skin );
          musicCheckBox.setChecked( SuperDroid.settings.musicEnabled );
          musicCheckBox.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
                  boolean enabled = musicCheckBox.isChecked();
                  SuperDroid.settings.musicEnabled = enabled;
                  if(enabled)
                  {
                	  Assets.playSound(Assets.clickSound);
                	  Assets.playMusic(Assets.menu);
                  }
                  else
                  {                	                   
                	  Assets.menu.stop();
                  }
              }
          } );
          layout.add(musicCheckBox );
          table.add(" Music ").left().padLeft(10);
          layout.row();
          
       // range is [0.0,1.0]; step is 0.1f
          Slider volumeSlider = new Slider( 0f, 1f, 0.1f, skin );
          volumeSlider.setValue( SuperDroid.settings.soundEffectVolume );
          volumeSlider.setValueChangedListener( new ValueChangedListener() {
              @Override
              public void changed(
                  Slider slider,
                  float value )
              {
                  SuperDroid.settings.soundEffectVolume = value;                                   
              }
          } );
          
          layout.add(volumeSlider );
          table.add(" Sound Effect Volume ").left().padLeft(10);
          layout.row();
          
          Slider musicVolumeSlider = new Slider( 0f, 1f, 0.1f, skin );
          musicVolumeSlider.setValue( SuperDroid.settings.musicVolume );
          musicVolumeSlider.setValueChangedListener( new ValueChangedListener() {
              @Override
              public void changed(
                  Slider slider,
                  float value )
              {
                  SuperDroid.settings.musicVolume = value;      
                  Assets.music.setVolume( value );
                  Assets.creditsMusic.setVolume( value );
                  Assets.menu.setVolume( value );
              }
          } );
          
          layout.add( musicVolumeSlider );
          table.add(" Music Volume ").left().padLeft(10);
          layout.row();      
          
          characterImage = new Image(new TextureRegion(Assets.characters.get(SuperDroid.settings.characterID), 0, 0, 56, 80), Scaling.stretch);
          characterImage.setClickListener( new ClickListener() {
              @Override
              public void click(
                  Actor actor,
                  float x,
                  float y )
              {
            	  	 SuperDroid.settings.characterID++;
            	  	 if(SuperDroid.settings.characterID == Assets.characters.size)
            	  		 SuperDroid.settings.characterID = 0;
            	  	 
                  		System.out.println("Character changed");
                	  Assets.playSound(Assets.clickSound);       
                	  characterImage.setRegion(new TextureRegion(Assets.characters.get(SuperDroid.settings.characterID), 0, 0, 56, 80));
              }
          } );
                              
          layout.add(characterImage).colspan(2).padTop(10);
          
          //layout.left().top().pad(40);
          layout.center();
          
                                    
		
	}   
}