package net.rastro.superdroid.view;

import net.rastro.superdroid.Settings;
import net.rastro.superdroid.model.World;

import com.badlogic.gdx.Application.ApplicationType;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

public class OnscreenControlRenderer {
        World map;
        SpriteBatch batch;
        TextureRegion dpad;
        TextureRegion left;
        TextureRegion right;
        TextureRegion jump;
        TextureRegion cubeControl;
        TextureRegion cubeFollow;
        TextureRegion pauseBtn;
        TextureRegion audioBtnOn;
        TextureRegion audioBtnOff;
        
        private int ww;
        private int hh;

        public OnscreenControlRenderer (World map) {
                this.map = map;             
        }
        
        public void setwwHH(int width, int height)
        {
        	ww=width;
        	hh=height;
        	loadAssets();
        }

        private void loadAssets () {
                Texture texture = new Texture(Gdx.files.internal("images/controls.png"));
                TextureRegion[] buttons = TextureRegion.split(texture, 64, 64)[0];
                left = buttons[0];
                right = buttons[1];
                jump = buttons[2];
                cubeControl = buttons[3];
                cubeFollow = TextureRegion.split(texture, 64, 64)[1][2];
                dpad = new TextureRegion(texture, 0, 64, 128, 128);
                
                Texture pauseBtnTexture = new Texture(Gdx.files.internal("images/pauseBtn.png"));
        		Texture audioBtnOnTexture = new Texture(Gdx.files.internal("images/audio_button_on.png"));
        		Texture audioBtnOffTexture = new Texture(Gdx.files.internal("images/audio_button_off.png"));
                pauseBtn = new TextureRegion(pauseBtnTexture,0,0,48,48);
                audioBtnOn = new TextureRegion(audioBtnOnTexture,0,0,48,48);
                audioBtnOff = new TextureRegion(audioBtnOffTexture,0,0,48,48);
                
                batch = new SpriteBatch();
                batch.getProjectionMatrix().setToOrtho2D(0, 0, ww, hh);//480x320
        }

        public void render () {
        	
        	batch.begin();
        	batch.draw(pauseBtn, 10, hh - 58);
        	if(Settings.soundEnabled)
        		batch.draw(audioBtnOn, 68, hh - 58);
        	else
        		batch.draw(audioBtnOff, 68, hh - 58);
        	
                if (Gdx.app.getType() == ApplicationType.Android){
                	batch.draw(dpad, 0, 0);
                	batch.draw(jump, ww - 64, 0);
                }
                batch.end();
        }

        public void dispose () {
                dpad.getTexture().dispose();
                pauseBtn.getTexture().dispose();
        		audioBtnOn.getTexture().dispose();
                batch.dispose();
        }
}

