package net.rastro.superdroid.view;

import net.rastro.superdroid.Assets;
import net.rastro.superdroid.SuperDroid;
import net.rastro.superdroid.controller.Animation;
import net.rastro.superdroid.model.Apple;
import net.rastro.superdroid.model.Block;
import net.rastro.superdroid.model.Bob;
import net.rastro.superdroid.model.Coin;
import net.rastro.superdroid.model.Ladder;
import net.rastro.superdroid.model.Laser;
import net.rastro.superdroid.model.Lift;
import net.rastro.superdroid.model.MovingSpike;
import net.rastro.superdroid.model.World;
import net.rastro.superdroid.model.gosign;
import net.rastro.superdroid.model.missile;
import net.rastro.superdroid.model.rockets;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.NinePatch;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.graphics.glutils.ImmediateModeRenderer20;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector3;

public class WorldRenderer {

	public static final float CAMERA_WIDTH = 15f;
	public static final float CAMERA_HEIGHT = 10f;
	
	public static Rectangle playAgainBtn = new Rectangle(10,10,10,10);

	private World world;
	public OrthographicCamera cam;

	/** for debug rendering **/
	ShapeRenderer debugRenderer = new ShapeRenderer();
	ImmediateModeRenderer20 renderer = new ImmediateModeRenderer20(false, true, 0);


	TextureRegion background;


	private SpriteBatch spriteBatch;		
	private boolean debug = false;
	private int width;
	private int height;
	private float ppuX;	// pixels per unit on the X axis
	private float ppuY;	// pixels per unit on the Y axis
	private float sx;
    private float ex;
    private float sy;
 	private float ey;
    
	public void setSize (int w, int h) {
		this.width = w;
		this.height = h;
		ppuX = (float)width / CAMERA_WIDTH;
		ppuY = (float)height / CAMERA_HEIGHT;
	}

	public WorldRenderer(World world, boolean debug) {
		this.world = world;
		this.cam = new OrthographicCamera(CAMERA_WIDTH, CAMERA_HEIGHT);
		this.cam.position.set(CAMERA_WIDTH / 2f, CAMERA_HEIGHT / 2f, 0);
		//this.cam.update();
		this.debug = debug;
		spriteBatch = new SpriteBatch();		
		renderBackground();
	}
	
	public void renderBackground () {
		spriteBatch.disableBlending();
		spriteBatch.begin();
		spriteBatch.draw(Assets.backgroundRegion, cam.position.x - CAMERA_WIDTH /2, cam.position.y - CAMERA_HEIGHT/2, CAMERA_WIDTH,
				CAMERA_HEIGHT);
		spriteBatch.end();
}


	Vector3 lerpTarget = new Vector3();

	public void render(float deltaTime) {					
		Bob bob = world.getBob();	
		
		if(bob.getPosition().x>cam.position.x  && cam.position.x <= world.WORLD_WIDTH-CAMERA_WIDTH /2) 
			cam.position.x=bob.getPosition().x;				
		else if( bob.getPosition().x<cam.position.x-1f && cam.position.x > (CAMERA_WIDTH/2))
			cam.position.x=bob.getPosition().x+1f;	
		
		if(bob.getPosition().y>cam.position.y && cam.position.y <= world.WORLD_HEIGHT-CAMERA_HEIGHT /2) 
			cam.position.y=bob.getPosition().y;				
		else if( bob.getPosition().y<cam.position.y-1f && cam.position.y > (CAMERA_HEIGHT/2)+0.1f)
			cam.position.y=bob.getPosition().y+1f;	
			
		
		cam.update();
		spriteBatch.setProjectionMatrix(cam.combined);
		
		renderBackground();	
		drawBackgroundBlocks();
		renderLaserBeams();
		spriteBatch.enableBlending();
		spriteBatch.begin();			
			drawBlocks();			
			drawLasers();
			drawApples();
			drawBob();	
			drawOverBob();
				//restartMenu();
		spriteBatch.end();
		if (debug)
			drawDebug();
		
		/*if(bob.getState() == Bob.DYING)
		{
			restartMenu(300, 150, bob.dieTimeElapsed);
		}*/
		
		/*if(world.getGameState() == World.GAME_FINISH)
			levelComplete(300, 150, bob.dieTimeElapsed);*/
		drawText("High Score: " + SuperDroid.settings.levelScores[world.getLevelId()-1] + "   Bonus: " + world.countDownTimer + "   Coins: " + world.coinsCollected + "/" + world.coinCount);
		if(world.getGameState() == World.GAME_READY)
			drawTextInMiddle("Press the screen or any key to begin playing!");
				
	}
	
	private void drawOverBob()
	{
		for (missile missile : world.getMissiles())
		{
			TextureRegion keyFrame;	
			if(missile.getState() == missile.STATE_FLYING)				
				spriteBatch.draw(Assets.missileAnimation.getKeyFrame(missile.getStateTime(), Animation.ANIMATION_LOOPING), (missile.getPosition().x ), missile.getPosition().y , missile.SIZE_W , missile.SIZE_H, 0.5f, 0.5f, 0.5f, 1, missile.getVel().angle()-90);
			if(missile.getState() == missile.STATE_EXPLODING)		
				spriteBatch.draw(Assets.explosionAnimation.getKeyFrame(missile.getStateTime(), Animation.ANIMATION_NONLOOPING), (missile.getPosition().x ), missile.getPosition().y , 1f , 1f);
			/*if(missile.getState() == missile.STATE_READY)				
				spriteBatch.draw(Assets.missileAnimation.getKeyFrame(missile.getStateTime(), Animation.ANIMATION_NONLOOPING), (missile.getPosition().x ), missile.getPosition().y , missile.SIZE_W , missile.SIZE_H, 0.5f, 0.5f, 0.5f, 1, missile.getVel().angle()-90);
			 	*/
		}
	}
	
	private void drawText(String str)
	{
		float textWidth = Assets.font.getBounds(str).width;
		spriteBatch.getProjectionMatrix().setToOrtho2D(0, 0, width, height);//480x320
		spriteBatch.begin();
		Assets.font.draw(spriteBatch, str, width - textWidth, height - 20);	
		spriteBatch.end();
	}
	

	private void drawTextInMiddle(String str)
	{
		float textWidth = Assets.font.getBounds(str).width;
		spriteBatch.getProjectionMatrix().setToOrtho2D(0, 0, width, height);//480x320
		spriteBatch.begin();
		Assets.font.draw(spriteBatch, str, (width - textWidth)/2, height/2);	
		spriteBatch.end();
	}
	
	private void levelComplete(float patchW, float patchH, float dieTime)
	{		
		//NinePatch patch = new NinePatch(new Texture(Gdx.files.internal("images/menuskin.png")), 8, 8, 8, 8);	
		//ScalableNinePatch sPatch = new ScalableNinePatch(patch);
		spriteBatch.getProjectionMatrix().setToOrtho2D(0, 0, width, height);//480x320
		spriteBatch.begin();
		NinePatch sPatch = new NinePatch(new Texture(Gdx.files.internal("images/menuskin.png")), 8, 8, 8, 8);
		
		sPatch.draw(spriteBatch, width/2-(patchW)/2, height/2-patchH/2, patchW, patchH);
		String str = "LEVEL COMPLETE!";
		float textWidth = Assets.font.getBounds(str).width;
		Assets.font.draw(spriteBatch, str, (width/2-(textWidth/2)), height/2+patchH/2-10);	
		
		String score = "Score: " + world.highScore;
		float scoreWidth = Assets.font.getBounds(score).width;
		float scoreHeight = Assets.font.getBounds(score).height;		
		float scorebtnx=(width/2-(scoreWidth/2));
		float scorebtny=height/2+patchH/2-scoreHeight-20;
		Assets.font.draw(spriteBatch, score, scorebtnx, scorebtny);	
		
		
		String playStr = "Play Again";
		float playtextWidth = Assets.font.getBounds(playStr).width;
		float playtextHeight = Assets.font.getBounds(playStr).height;		
		float btnx=(width/2-(playtextWidth/2));
		float btny=height/2+patchH/2-playtextHeight-40;
		Assets.font.draw(spriteBatch, playStr, btnx, btny);	
		playAgainBtn = new Rectangle(btnx, btny-playtextHeight, playtextWidth, playtextHeight);		//the XY starts from top left corner, instead of bottom ASSHOLES
		
		
		spriteBatch.end();		
		
		
		/*debugRenderer.begin(ShapeType.Rectangle);
		debugRenderer.setColor(new Color(0, 1, 0, 1));
		debugRenderer.rect(playAgainBtn.x, playAgainBtn.y, playAgainBtn.width, playAgainBtn.height);
		debugRenderer.rect(touchPoint.x, touchPoint.y, playAgainBtn.width, playAgainBtn.height);
		debugRenderer.rect(width/2-(patchW)/2, height/2-patchH/2,  patchW, patchH);
		debugRenderer.end();*/
		
	}

	
	private void restartMenu(float patchW, float patchH, float dieTime)
	{		
		//NinePatch patch = new NinePatch(new Texture(Gdx.files.internal("images/menuskin.png")), 8, 8, 8, 8);	
		//ScalableNinePatch sPatch = new ScalableNinePatch(patch);
		spriteBatch.getProjectionMatrix().setToOrtho2D(0, 0, width, height);//480x320
		spriteBatch.begin();
		NinePatch sPatch = new NinePatch(new Texture(Gdx.files.internal("images/menuskin.png")), 8, 8, 8, 8);
		
		sPatch.draw(spriteBatch, width/2-(patchW)/2, height/2-patchH/2, patchW, patchH);
		String str = "GAME OVER!";
		float textWidth = Assets.font.getBounds(str).width;
		Assets.font.draw(spriteBatch, str, (width/2-(textWidth/2)), height/2+patchH/2-10);	
		
		String playStr = "Play Again";
		float playtextWidth = Assets.font.getBounds(playStr).width;
		float playtextHeight = Assets.font.getBounds(playStr).height;		
		float btnx=(width/2-(playtextWidth/2));
		float btny=height/2+patchH/2-playtextHeight-20;
		Assets.font.draw(spriteBatch, playStr, btnx, btny);	
		playAgainBtn = new Rectangle(btnx, btny-playtextHeight, playtextWidth, playtextHeight);		//the XY starts from top left corner, instead of bottom ASSHOLES
		
		
		spriteBatch.end();		
		
		
		/*debugRenderer.begin(ShapeType.Rectangle);
		debugRenderer.setColor(new Color(0, 1, 0, 1));
		debugRenderer.rect(playAgainBtn.x, playAgainBtn.y, playAgainBtn.width, playAgainBtn.height);
		debugRenderer.rect(touchPoint.x, touchPoint.y, playAgainBtn.width, playAgainBtn.height);
		debugRenderer.rect(width/2-(patchW)/2, height/2-patchH/2,  patchW, patchH);
		debugRenderer.end();*/
		
	}
	
	private void drawBackgroundBlocks() 
	{
		spriteBatch.disableBlending();
		spriteBatch.begin();
		TextureRegion blockSurface = null;
		for (Block block : world.getBackgroundBlocks()) {			
			//spriteBatch.draw(blockTexture, (block.getPosition().x * ppuX), block.getPosition().y * ppuY, Block.SIZE * ppuX, Block.SIZE * ppuY);
			if(!block.amIActive()) continue;
			if(block.getType() == Block.STONE) blockSurface = Assets.backStone;
			if(block.getType() == Block.DIRT) blockSurface = Assets.backDirt;			
			spriteBatch.draw(blockSurface, (block.getPosition().x ), block.getPosition().y , Block.SIZEX , block.getSizeY());
		}
		spriteBatch.end();
	}

	private void drawBlocks() {
		TextureRegion blockSurface = null;		
		for (Block block : world.getBlocks()) {			
			//spriteBatch.draw(blockTexture, (block.getPosition().x * ppuX), block.getPosition().y * ppuY, Block.SIZE * ppuX, Block.SIZE * ppuY);
			if(!block.amIActive()) continue;
			if(block.getType() == Block.STONE) blockSurface = Assets.stone;
			if(block.getType() == Block.GRASS) blockSurface = Assets.grass;
			if(block.getType() == Block.DIRT) blockSurface = Assets.dirt;
			if(block.getType() == Block.ICE) blockSurface = Assets.ice;
			if(block.getType() == Block.SPIKES) blockSurface = Assets.spikes;
			if(block.getType() == Block.GRASS_DISOLVE && !block.doIDissolve() && block.amIActive()) blockSurface = Assets.grass;
			if(block.getType() == Block.GRASS_DISOLVE && block.doIDissolve() && block.amIActive())
				blockSurface = Assets.grassDissolveAnim.getKeyFrame(block.getStateTime(), Animation.ANIMATION_NONLOOPING);	
			if(block.getType() == Block.DIRT_DISSOLVE && !block.doIDissolve() && block.amIActive()) blockSurface = Assets.dirt;
			if(block.getType() == Block.DIRT_DISSOLVE && block.doIDissolve() && block.amIActive())
				blockSurface = Assets.dirtDissolveAnim.getKeyFrame(block.getStateTime(), Animation.ANIMATION_NONLOOPING);
			if(block.getType() == Block.STONE_DISSOLVE && !block.doIDissolve() && block.amIActive()) blockSurface = Assets.stone;
			if(block.getType() == Block.STONE_DISSOLVE && block.doIDissolve() && block.amIActive())
				blockSurface = Assets.stoneDissolveAnim.getKeyFrame(block.getStateTime(), Animation.ANIMATION_NONLOOPING);
			if(block.getType() == Block.ICE_DISSOLVE && !block.doIDissolve() && block.amIActive()) blockSurface = Assets.ice;
			if(block.getType() == Block.ICE_DISSOLVE && block.doIDissolve() && block.amIActive())
				blockSurface = Assets.iceDissolveAnim.getKeyFrame(block.getStateTime(), Animation.ANIMATION_NONLOOPING);
			spriteBatch.draw(blockSurface, (block.getPosition().x ), block.getPosition().y , Block.SIZEX , block.getSizeY());
		}
		for (gosign sign : world.getGoSign()) {
			//spriteBatch.draw(blockTexture, (block.getPosition().x * ppuX), block.getPosition().y * ppuY, Block.SIZE * ppuX, Block.SIZE * ppuY);
			TextureRegion signSurface = null;
			if(sign.getType() == gosign.Types.RIGHT) signSurface = Assets.goSign;
			if(sign.getType() == gosign.Types.LEFT) signSurface = Assets.goSignLeft;
			if(sign.getType() == gosign.Types.UP) signSurface = Assets.goSignUp;
			if(sign.getType() == gosign.Types.DOWN) signSurface = Assets.goSignDown;
			spriteBatch.draw(signSurface, (sign.getPosition().x ), sign.getPosition().y , sign.SIZE , sign.SIZE);
		}
		for (rockets rockets : world.getRockets()) {
			//spriteBatch.draw(blockTexture, (block.getPosition().x * ppuX), block.getPosition().y * ppuY, Block.SIZE * ppuX, Block.SIZE * ppuY);
			spriteBatch.draw(Assets.rocketLauncher, (rockets.getPosition().x ), rockets.getPosition().y , rockets.SIZE , rockets.SIZE);
			
		}
		for (Coin coin : world.getCoins())
		{
			if(coin.isActive())
				spriteBatch.draw(Assets.coinSpinning.getKeyFrame(coin.stateTime, Animation.ANIMATION_LOOPING), (coin.getPosition().x ), coin.getPosition().y , coin.SIZE , coin.SIZE);
		}
		for (Ladder ladder : world.getLadders())
		{
			spriteBatch.draw(Assets.ladderTexture, (ladder.getPosition().x ), ladder.getPosition().y , ladder.SIZE , ladder.SIZE);
		}
		for(Lift lift : world.getLifts())
		{
			spriteBatch.draw(Assets.liftAnimation.getKeyFrame(lift.getTime(), Animation.ANIMATION_LOOPING), (lift.getPosition().x ), lift.getPosition().y , lift.SIZE_W , lift.SIZE_H);
		}
		for(MovingSpike movingSpike : world.getMovingSpikes())
		{
			spriteBatch.draw(Assets.movingSpikeTexture, (movingSpike.getPosition().x ), movingSpike.getPosition().y , movingSpike.SIZE , movingSpike.SIZE);
		}
		
			spriteBatch.draw(Assets.finishSign, (world.getEnd().getPosition().x ), world.getEnd().getPosition().y , world.getEnd().SIZE , world.getEnd().SIZE);
		
	}
	
	private void drawApples()
	{
		TextureRegion appleTexture = null;
		for(Apple apple : world.getApples())
		{
			if(!apple.amIActive()) continue;
			
			if(apple.amIDying())
				appleTexture = Assets.appleDying.getKeyFrame(apple.getTime(), Animation.ANIMATION_NONLOOPING);
			else
				appleTexture = Assets.appleWalking.getKeyFrame(apple.getTime(), Animation.ANIMATION_LOOPING);
			
			if(apple.isWalkingLeft())
			{
				spriteBatch.draw(appleTexture, ((apple.getPosition().x+Apple.SIZE) ), apple.getPosition().y , -apple.SIZE , apple.SIZE );
			}
			else
			{
				spriteBatch.draw(appleTexture, (apple.getPosition().x ), apple.getPosition().y , apple.SIZE , apple.SIZE);	
			}			
		}
	}
	
	private void drawLasers() {
		for (Laser laser : world.getLasers()) {			
			
			if(laser.getType() == Laser.GROUND)
				spriteBatch.draw(Assets.laserGun, (laser.getPosition().x ), laser.getPosition().y , laser.SIZE , laser.SIZE);
			if(laser.getType() == Laser.CEILING)
				spriteBatch.draw(Assets.laserGun, (laser.getPosition().x ), laser.getPosition().y + laser.SIZE , laser.SIZE , -laser.SIZE);
			if(laser.getType() == Laser.LEFT)			
				spriteBatch.draw(Assets.laserGunRotated, (laser.getPosition().x ), laser.getPosition().y, laser.SIZE , laser.SIZE);
			if(laser.getType() == Laser.RIGHT)			
				spriteBatch.draw(Assets.laserGunRotated, (laser.getPosition().x )+laser.SIZE, laser.getPosition().y, -laser.SIZE , laser.SIZE);			
		}
	}
	
	private void renderLaserBeams () {
        //cam.update(false);
        renderer.begin(cam.combined, GL10.GL_LINES);
        
        for (int i = 0; i < world.getLasers().size; i++) {        	
             Laser laser = world.getLasers().get(i);        
             if(!laser.isActive()) continue;
			if(laser.getType() == Laser.GROUND)
             {
                sx = laser.getPosition().x+laser.SIZE/2; sy = laser.getPosition().y;
                ex = laser.getPosition().x+laser.SIZE/2; ey = laser.getPosition().y+laser.getLength();
             }
			if(laser.getType() == Laser.CEILING)
            {
               sx = laser.getPosition().x+laser.SIZE/2; sy = laser.getPosition().y+laser.SIZE;
               ex = laser.getPosition().x+laser.SIZE/2; ey = laser.getPosition().y-laser.getLength();
            }
			if(laser.getType() == Laser.LEFT)
            {
               sx = laser.getPosition().x; sy = laser.getPosition().y+laser.SIZE/2;
               ex = laser.getPosition().x+laser.getLength(); ey = laser.getPosition().y+laser.SIZE/2;
            }
			if(laser.getType() == Laser.RIGHT)
            {
               sx = laser.getPosition().x+laser.SIZE; sy = laser.getPosition().y+laser.SIZE/2;
               ex = laser.getPosition().x-laser.getLength(); ey = laser.getPosition().y+laser.SIZE/2;
            }
                renderer.color(1, 0, 0, 1);
                renderer.vertex(sx, sy, 0);
                renderer.color(1, 0, 0, 1);
                renderer.vertex(ex, ey, 0);
        }
        renderer.end();
}
		

	private void drawBob() {
		Bob bob = world.getBob();		
		
		
		TextureRegion keyFrame;		
		switch (bob.state)
		{
		case Bob.IDLE:
			keyFrame = Assets.bobIdle.getKeyFrame(bob.stateTime, Animation.ANIMATION_LOOPING);
			break;
		case Bob.JUMPING:
			keyFrame = Assets.bobJump.getKeyFrame(bob.stateTime, Animation.ANIMATION_NONLOOPING);
			break;	
		case Bob.FALLING:
			keyFrame = Assets.bobFall;
			break;	
		case Bob.WALKING:
			keyFrame = Assets.bobWalk.getKeyFrame(bob.stateTime, Animation.ANIMATION_LOOPING);			
			break;	
		case Bob.WALKING_LADDER:
			keyFrame = Assets.bobLadders.getKeyFrame(bob.stateTime, Animation.ANIMATION_LOOPING);			
			break;	
		case Bob.DYING:
			keyFrame = Assets.bobDying.getKeyFrame(bob.stateTime, Animation.ANIMATION_LOOPING);			
			break;	
		default:
				keyFrame = Assets.bobIdle.getKeyFrame(bob.stateTime, Animation.ANIMATION_LOOPING);
		}		
		
		if(bob.facingLeft)
		{
			spriteBatch.draw(keyFrame, ((bob.getPosition().x+Bob.BOB_WIDTH) ), bob.getPosition().y , -Bob.BOB_WIDTH , Bob.BOB_HEIGHT+0.2f );
		}
		else
		{
			spriteBatch.draw(keyFrame, bob.getPosition().x , bob.getPosition().y , Bob.BOB_WIDTH , Bob.BOB_HEIGHT+0.2f );	
		}
		
	}

	private void drawDebug() {
		// render blocks
		debugRenderer.setProjectionMatrix(cam.combined);
		debugRenderer.begin(ShapeType.Rectangle);
		debugRenderer.setColor(new Color(0, 1, 0, 1));
		/*for (Block block : world.getBlocks()) {
			Rectangle rect = block.getBounds();
			float x1 = block.getPosition().x + rect.x;
			float y1 = block.getPosition().y + rect.y;
			debugRenderer.setColor(new Color(1, 0, 0, 1));
			//debugRenderer.rect(x1, y1, rect.width, rect.height);
			debugRenderer.rect(rect.x,rect.y,rect.width, rect.height);
		}
		// render Bob
		Bob bob = world.getBob();
		Rectangle rect = bob.getBounds();
		float x1 = bob.getPosition().x + rect.x;
		float y1 = bob.getPosition().y + rect.y;		
		//debugRenderer.rect(x1, y1, rect.width, rect.height);
		debugRenderer.rect(rect.x,rect.y,rect.width, rect.height);*/
		debugRenderer.rect(playAgainBtn.x, playAgainBtn.y, playAgainBtn.width, playAgainBtn.height);
		debugRenderer.end();
	}
}